/* eslint max-lines: off */

import ComponentEvent from './event.js?v=1.12.1'
import Logger from './logging.js?v=1.12.1'
import {cleanContextEval} from './utils.js?v=1.12.1'
import theme from './theme.js?v=1.12.1'

const DATASET_DATA_URL           = 'url'
const DATASET_COMPONENT_DATA_KEY = 'data'

/**
 * The base component class which implements most of the web components magic
 * and inherited by each component itself.
 *
 * All components can be :ref:`accessed through the meta object <access components>`.
 */

class BaseComponent extends HTMLElement {

    constructor() {
        super()

        /*
         * Setup logger.
         */

        this.logger = new Logger(this.localName)
        this.logger.debug(`📦 Initialising new "${this.localName}" component…`)

        /*
         * Attach a shadow DOM tree to the web component, as we want proper
         * encapsualation.
         */

        this.attachShadow({'mode': 'open'})

        /*
         * Initialise an empty loop context, which will later be populated by
         * the renderLoopElement() and injected by the buildContext() method.
         */

        this.loopContext = {}
    }

    /**
     * Callback which is automatically called by the browser when the component
     * is inserted into the DOM. This will trigger the :js:meth:`reloadData()`
     * and :js:meth:`fetchData()` methods.
     */

    connectedCallback() {
        /*
         * Reload & fetch data.
         */

        this.reloadData()

        if (this.dataset[DATASET_DATA_URL])
            this.fetchData()

        /*
         * Set theme mode on component, so that it can be queries in the
         * component itself.
         */

        this.setAttribute('theme-mode', theme.currentMode)

        document.addEventListener('switchThemeMode', () => {
            this.setAttribute('theme-mode', theme.currentMode)
        })
    }

    /* eslint-disable class-methods-use-this */

    /**
     * The default data, which is used when no data attribute is defined.
     *
     * .. hint::
     *
     *      This method can be overwritten in the **concrete component** classes
     *      in case they need some default data.
     *
     * @return  {Object}    The default data
     */

    get defaultData() {
        return ''
    }

    /**
     * The custom context for the usage in templates, injected in the
     * :js:meth:`buildContext()` method.
     *
     * .. hint::
     *
     *      This method can be overwritten in the **concrete component** classes
     *      in case they need some custom context.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        return {}
    }

    /* eslint-enable class-methods-use-this */

    /**
     * Reload the data from the ``data-data`` attribute.
     *
     * In case no data is defined in the ``data-data`` attribute, the default
     * data from the :js:meth:`defaultData()` getter is loaded instead.
     *
     * This will delegate the data update to :js:meth:`updateData()`.
     *
     * @return  {Object}    The data
     */

    reloadData() {
        if (this.dataset[DATASET_COMPONENT_DATA_KEY])
            this.updateData(JSON.parse(this.dataset[DATASET_COMPONENT_DATA_KEY]))
        else
            this.updateData(this.defaultData)
    }

    /**
     * API to fetch the data from a REST backend with the :js:attr:`dataUrl`
     * URL. If no :js:attr:`dataUrl` is defined, the URL is read from the
     * element's ``data-url`` attribute.
     *
     * This will delegate the data update to :js:meth:`updateData()`.
     *
     * @param   {String} dataURL    The data URL
     *
     * @return  {Promise}           The promise with the data
     */

    fetchData(dataURL) {
        const url = dataURL ? dataURL : this.dataset[DATASET_DATA_URL]

        this.logger.debug('   💾 Fetching data…')
        this.logger.debug('      URL: ', url)

        return fetch(url)
            .then(response => response.json())
            .then(data => this.updateData(data))
    }

    /**
     * API to update the data and re-trigger the rendering via
     * :js:meth:`render()`.
     *
     * @param   {Object} data   The data
     *
     * @return  {Object}        The data
     */

    updateData(data) {
        this.data = data

        this.render()

        if (typeof this.formData !== 'undefined')
            this.dispatchEvent(new ComponentEvent('updateFormData'))

        return data
    }

    /**
     * Build the context for the usage in templates, such as the text &
     * attribute interpolations, for loop expressions & conditional validations.
     *
     * By default the context includes these keys:
     *
     *  - ``attributes``: The component's attributes
     *  - ``component``: The component's name
     *  - ``data``: The component's data
     *
     * The method will also inject the loop context set by
     * :js:meth:`renderLoopElement()` and the :js:meth:`customContext()`.
     *
     * @param   {HTMLElement} element   The HTML element
     *
     * @return  {Object}                The context
     */

    // eslint-disable-next-line no-unused-vars
    buildContext(element) {
        return {
            'attributes': this.componentAttributes,
            'component': this.localName,
            'data': this.data,
            ...this.loopContext,    // This is updated by the renderLoopElement() method.
            ...this.customContext,  // This is optionally set by the concrete web component.
        }
    }

    /**
     * Build the component attributes.
     *
     * @return  {Object}    The component attributes
     */

    buildComponentAttributes() {
        const attributes = {}
        Object.values(this.attributes).forEach(attribute => {
            attributes[attribute.name] = attribute.value || attribute.specified
        })
        return attributes
    }

    /**
     * Reflect some attributes to element attributes.
     *
     * This is achieved by looping over all :js:attr:`elements` & :js:attr:`attributes`, and by
     * reflecting the values received by :js:attr:`valueCallback` to the target element received by
     * :py:attr:`targetCallback`.
     *
     * .. note::
     *
     *      The :js:attr:`targetCallback` can be used to define the target element to which the
     *      attributes should be reflected to. For example:
     *
     *      .. code-block:: javascript
     *
     *          element => element.querySelector('div#target')
     *
     *      The :js:attr:`valueCallback` can be used to lookup & define the value which should be
     *      used for the reflected attribute. For example:
     *
     *      .. code-block:: javascript
     *
     *          (element, attribute) => this.exampleGetter(attribute)
     *
     *      Please note, if the returned value is boolean ``false``, the attribute will not be set
     *      and skipped.
     *
     * @param   {Array} attributes          The list of attributes to reflect
     * @param   {NodeList} elements         The list of elements to loop over
     * @param   {Function} targetCallback   The callback which returns the target element
     * @param   {Function} valueCallback    The callback which returns the attribute value
     */

    reflectAttributes(attributes, elements, targetCallback, valueCallback) {
        this.logger.debug('      🔃 Reflecting attributes…')
        this.logger.debug('         Attributes: ', attributes)

        for (const element of elements) {
            const target = targetCallback(element)
            this.logger.debug('         Element & Target: ', element, target)

            for (const attribute of attributes) {
                const value = valueCallback(element, attribute)

                if (value !== false) {
                    this.logger.debug('           Setting attribute: ', `${attribute}="${value}"`)
                    target.setAttribute(attribute, value)
                }
            }
        }
    }

    /**
     * Reflect some component attributes to element attributes.
     *
     * This is achieved by looping over all :js:attr:`elements` and by reflecting all
     * :js:attr:`attributes` from the component to them.
     *
     * @param   {Array} attributes      The list of attributes to reflect
     * @param   {NodeList} elements     The list of elements to reflect the attributes to
     */

    reflectComponentAttributes(attributes, elements) {
        return this.reflectAttributes(
            attributes,
            elements,
            element => element,
            (element, attribute) => {
                if (this.attributes[attribute])
                    return this.attributes[attribute].value
                return false
            },
        )
    }

    /**
     * Reflect some data properties to element attributes.
     *
     * This is achieved by looping over all :js:attr:`elements` and by reflecting all
     * :js:attr:`attributes` from the element's data to the element received by the
     * :js:attr:`targetCallback`.
     *
     * @param   {Array} attributes          The list of attributes to reflect
     * @param   {NodeList} elements         The list of elements to loop over
     * @param   {Function} targetCallback   The callback which returns the target element
     */

    reflectDataProperties(attributes, elements, targetCallback) {
        return this.reflectAttributes(
            attributes,
            elements,
            targetCallback,
            (element, attribute) => (element.data && element.data.value[attribute]) || false,
        )
    }

    /**
     * Interpolate all attributes of an element via :js:func:`interpolate()`.
     *
     * @param   {HTMLElement} element   The HTML element
     * @param   {Object} context        The context
     */

    interpolateAttributes(element, context) {
        this.logger.debug('      📝 Interpolating element attributes…')
        element.getAttributeNames().forEach(name => {
            element.setAttribute(name, element.getAttribute(name).interpolate(context))
        })
    }

    /**
     * Interpolate the text content an element via :js:func:`interpolate()`.
     *
     * @param   {HTMLElement} element   The HTML element
     * @param   {Object} context        The context
     */

    interpolateTextContent(element, context) {
        this.logger.debug('      📝 Interpolating text content…')
        element.textContent = element.textContent.interpolate(context)
    }

    /**
     * Handle an element with an event listener, namely an element with a
     * ``t-on`` attribute and the following value / syntax:
     *
     * .. code-block::
     *
     *      "{originalEvent}:{customEvent}"
     *
     * The method will then register an event listener for the original event,
     * which dispatches a new Translucent-specific :js:class:`ComponentEvent`.
     *
     * @param   {HTMLElement} element   The conditional element
     */

    handleEventListener(element) {
        for (const pair of element.getAttribute('t-on').split(' ')) {
            const on                       = pair.trim().split(':')
            let [originalType, customType] = on  // eslint-disable-line prefer-const

            if (typeof customType === 'undefined')
                customType = originalType

            this.logger.debug('      🗯 ️Handling event listener…')
            this.logger.debug('         Original Event:', originalType)
            this.logger.debug('         Custom Event:', customType)

            element.addEventListener(originalType, () => {
                this.logger.debug('🎯 Dispatching event callback…')
                this.logger.debug('   Type: ', customType)
                element.dispatchEvent(new ComponentEvent(customType))
            })
        }

        element.removeAttribute('t-on')
    }

    /**
     * Validate a conditional element, namely an element with a ``t-if``
     * attribute. Evaluate the expression via :js:func:`cleanContextEval()`.
     *
     * @param   {HTMLElement} element   The conditional element
     * @param   {Object} context        The context
     *
     * @return  {Boolean}               The condition success
     */

    validateConditionalElement(element, context) {
        /*
         * Read the condition from the element. When the condition is read,
         * we no longer need it in the element and we certainly don't want it
         * in the DOM. Thus, remove it.
         */

        const condition = element.getAttribute('t-if')
        this.logger.debug('      🚦 Evaluating condition…')
        this.logger.debug('         Condition:', condition)

        element.removeAttribute('t-if')

        /*
         * Evaluate the condition in a clean context.
         */

        const result = cleanContextEval(`return ${condition}`, context)
        const bool   = Boolean(result)

        this.logger.debug('         Native Result: ', result)
        this.logger.debug('         Boolean Result: ', bool)

        return bool
    }

    /**
     * Render a loop element, namely handling an element with a ``t-for``
     * attribute. The loop element can return 0+ elements.
     *
     * @param   {HTMLElement}   element     The HTML element
     * @param   {Object}        context     The context
     *
     * @return  {Array}                     The generated elements based on the data
     */

    // eslint-disable-next-line max-lines-per-function, max-statements
    renderLoopElement(element, context) {
        /*
         * Loop through all attributes and find the first matching `t-for` or
         * `t-for-{name}`. Cache that in a var and remove the attribute from the
         * element, since we don't want that in the DOM.
         */

        let loopAttribute = null

        for (const attribute of element.attributes) {
            if (attribute.name === 't-for' || attribute.name.startsWith('t-for-')) {
                element.removeAttribute(attribute)
                loopAttribute = attribute
                break
            }
        }

        /*
         * The loop attribute includes all informations for the loop. It always
         * includes the expression to evaluate (attribute value). In case of the
         * `t-for-{name}` syntax, it also includes the name for the loop
         * variable. So extract those values and evaluate the expression in a
         * clean context.
         */

        const prefixLength = 6
        const loopVar      = loopAttribute.name.substring(prefixLength) || 'item'
        const expression   = loopAttribute.value
        const items        = cleanContextEval(`return ${expression}`, context)

        this.logger.debug('      🔄 Rendering loop element…')
        this.logger.debug('         Expression: ', expression)
        this.logger.debug('         Loop Variable: ', loopVar)
        this.logger.debug('         Items: ', items)

        /*
         * Since we've the items, we can render a new element for each item.
         * Because we need to access the item-specific data later (i.e.
         * condition valuation or interpolation), we've to store the
         * item-specific data to the item itself.
         */

        let clone           = null
        let children        = []
        const isArray       = items instanceof Array
        const {loopContext} = this

        Object.entries(items).forEach(([key, value]) => {
            clone                = element.cloneNode(true)
            clone.data           = isArray ? value : {key, value}
            loopContext[loopVar] = clone.data
            children             = [...children, this.renderElement(clone)]
            delete loopContext[loopVar]
        })

        return children
    }

    /**
     * Render a HTML element.
     *
     * This is where basically all the magic happens, or at least where the
     * magic starts. The :js:meth:`render()` method will initially pass the
     * :js:meth:`template()` to this method. This method will then recursively
     * call itself for each child found in the element.
     *
     * .. hint::
     *
     *      The method will delegate several actions to other methods.
     *
     *      - For each conditional child, the
     *        :js:meth:`validateConditionalElement()` method is invoked.
     *      - For each loop child, the rendering is passed to the
     *        :js:meth:`renderLoopElement()` method.
     *      - Event listeners are added via :js:meth:`handleEventListener`.
     *      - Text interpolations are handled via :js:meth:`interpolateTextContent()`.
     *      - Attribute interpolations are handled via :js:meth:`interpolateAttributes()`.
     *
     * @param   {HTMLElement} element   The HTML template element
     *
     * @return  {HTMLElement}           The rendered HTML element
     */

    // eslint-disable-next-line max-lines-per-function, max-statements
    renderElement(element) {
        this.logger.debug('   🖌 Rendering element…')
        this.logger.debug('      🖼 Element: ', element)

        /*
         * Building the interpolation & validation context for the element.
         */

        const context = this.buildContext(element)
        this.logger.debug('      🗄 Context: ', context)

        /*
         * Some elements are conditional, so ensure we only render the required
         * ones.
         */

        if (element.hasAttribute('t-if') && !this.validateConditionalElement(element, context))
            return null

        /*
         * Before we can start handling the element, we need some clones.
         *
         * First of all, we need clones as the same element can be used several
         * times in the DOM.
         *
         * Then this method is called recursively, once for each child in the
         * element. The most straight-forward way to do that is by:
         *
         *  1. Creating a new clone without children (a naked clone).
         *
         *  2. Creating a a separate clone of all children.
         *
         *  3. Rendering the children recursively and adding them to the naked
         *     clone again.
         */

        const clone = element.cloneNode(false)
        clone.data  = element.data

        const children = element.cloneNode(true).getChildNodes()

        /*
         * An element can add custom events via template literals (i.e. `t-on`),
         * so we need to handle them.
         */

        if (clone.hasAttribute('t-on'))
            this.handleEventListener(clone)

        /*
         * An element can use template literals (i.e. ${…}) in attributes to
         * use dynamic data values. Thus, we need to interpolate them here.
         */

        this.interpolateAttributes(clone, context)

        /*
         * Now we've to loop over all children and we need to differ between
         * them. Text nodes (i.e. node type 3) must be interpolated, but can
         * then directly be added to the clone. Loop nodes (i.e. nodes with a
         * 't-for' attribute) must be handled separately. The remaining nodes
         * will be recursively rendered again.
         */

        const textNode    = 3
        const commentNode = 8

        children.forEach(child => {
            if (child.nodeType === commentNode)
                return

            if (child.nodeType === textNode) {
                this.interpolateTextContent(child, context)
                clone.appendChildren([child])
            }

            else if (child.hasAttribute('t-for') || child.hasAttributesStartingWith('t-for-'))
                clone.appendChildren(this.renderLoopElement(child, context))

            else
                clone.appendChildren([this.renderElement(child)])
        })

        return clone
    }

    /**
     * Render the web component to the shadow DOM.
     *
     * This method is basically a wrapper for:
     *
     * - Clearing the :js:attr:`this.shadowRoot` inner HTML
     * - Adding the style of :js:meth:`style()` to the shadow root
     * - Render the template of :js:meth:`template()` via :js:meth:`renderElement()`
     * - Add the rendered template to the shadow root
     */

    render() {
        /*
         * Set the component attributes.
         */

        this.componentAttributes = this.buildComponentAttributes()

        /*
         * Cleanup old shadow DOM root.
         */

        this.shadowRoot.innerHTML = ''

        /*
         * Render the component CSS style to the shadow DOM root.
         */

        if (this.constructor.style) {
            const styleNode     = document.createElement('style')
            styleNode.textContent = this.constructor.style
            this.shadowRoot.append(styleNode)
        }

        /*
         * ES6 doesn't allow us to "simply" convert a string into a proper
         * node. Thus, we need to leverage the HTML5 <template> node for that.
         */

        const range   = document.createRange()
        const clone   = range.createContextualFragment(this.constructor.template)


        for (const child of clone.children)
            this.shadowRoot.append(this.renderElement(child))
    }

}

export default BaseComponent
