/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CCheckboxComponent extends BaseComponent {

    /**
     * The checked items, returned in a key-value object, where the object key
     * is the data value, and the object value is the data label.
     *
     * @return  {Object}    The checked items
     */

    get checked() {
        const checked = this.shadowRoot.querySelectorAll('input:checked:not(:disabled)')
        return [...checked].reduce((obj, el) => {
            obj[el.value] = el.dataset.label
            return obj
        }, {})
    }

    /**
     * The data of this component, which should be transferred when a form is
     * submitted.
     *
     * @return  {Object}    The form data
     */

    get formData() {
        const {name} = this.componentAttributes
        const values = Object.keys(this.checked)
        return this.data ? {[name]: values} : {[name]: values[0]}
    }

    /**
     * The custom context, which is used in the `template.html` file of the
     * c-checkbox component.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        const data = this.data ? {...this.data} : ''

        return {
            data,
        }
    }

    reflectInputAttributes() {
        const inputElements = this.shadowRoot.querySelectorAll('.input-wrapper')
        const attributes    = ['checked', 'disabled']

        if (inputElements) {
            for (const inputElement of inputElements) {
                const input = inputElement.querySelector('input')

                for (const attribute of attributes) {
                    if (inputElement.data && inputElement.data.value[attribute] === true)
                        input.setAttribute(attribute, attribute)
                }
            }
        }
    }

    /**
     * Reflect the input attributes after rendering.
     */

    render() {
        super.render()

        this.reflectComponentAttributes(
            [
                'checked',
                'disabled',
            ],
            this.shadowRoot.querySelectorAll('input'),
        )

        this.reflectDataProperties(
            [
                'checked',
                'disabled',
            ],
            this.shadowRoot.querySelectorAll('.input-wrapper'),
            element => element.querySelector('input'),
        )
    }

}

/*
 * HTML template & CSS style for c-checkbox.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CCheckboxComponent.template = `<fieldset>
    <div t-if="data" class="multioptions">
        <c-label t-if="attributes.label" class="label-text" for="\${name}" required="\${attributes.required}">
            \${attributes.label}
        </c-label>

        <div class="content-wrapper">
            <span t-for="data" class="input-wrapper">
                <input
                    type="checkbox"
                    id="\${item.value.value}"
                    name="\${item.value.name}"
                    value="\${item.value.value}"
                    data-label="\${item.value.label}"
                    t-on="change:updateFormData"
                >

                <label for="\${item.value.value}">
                    <span class="check-icon"></span>
                    <span class="check-text">\${item.value.label}</span>
                </label>
            </span>

            <div t-if="attributes.help" class="help">
                \${attributes.help}
            </div>
            <slot name="help" class="help"></slot>

            <div t-if="attributes.error" class="error">
                \${attributes.error}
            </div>
        </div>
    </div>

    <!-- Single Checkbox -->
    <span class="input-wrapper" t-if="!data">
        <input
                type="checkbox"
                id="\${attributes.value}"
                name="\${attributes.name}"
                t-on="change:updateFormData"
            >
        <label for="\${attributes.value}">
            <span class="check-icon"></span>
            <span class="check-text">
                <slot>Checkbox Label</slot>
                <span t-if="attributes.help" class="help">
                    \${attributes.help}
                </span>
            </span>

            <slot name="help" class="help"></slot>
        </label>
    </span>
</fieldset>`

CCheckboxComponent.style = `/* ============================================================ *
   GENERIC
 * ============================================================ */

/* -------------------- RESETS -------------------- */
*,
::before,
::after {
    box-sizing: border-box;
}

/*
 * Change the cursor on disabled, not-editable, or otherwise
 * inoperable elements in all browsers (opinionated).
 */

[aria-disabled="true"],
[disabled] {
    cursor: not-allowed;
}

/* ============================================================ *
   ELEMENTS
 * ============================================================ */

:host {
    --checkbox-size: 1rem;
    --check-size: 90%;
    --check-position-left: calc((var(--checkbox-size) - var(--check-size)) / 2);

    display: block;
    width: 100%;
    margin-bottom: var(--forms-margin);
}

fieldset {
    padding: 0;
    border: 0;
}

input {
    position: absolute;
    overflow: hidden;
    width: 1px;
    height: 1px;
    padding: 0;
    border: 0;
    margin: -1px;
    clip: rect(0, 0, 0, 0);
    visibility: inherit;
    white-space: nowrap;
}

.error {
    font-size: var(--forms-error-font-size);
}

.help {
    font-size: var(--forms-help-font-size);
    color: var(--forms-color-help);
    font-style: var(--forms-help-font-style);
    display: inline;
    width: 100%;
}

label {
    box-sizing: border-box;
    border: 0;
    margin: 0;
    font-family: inherit;
    position: relative;
    display: flex;
    flex-wrap: wrap;
    min-height: calc(0.5rem + var(--checkbox-size));
    cursor: pointer;
    align-items: baseline;
    user-select: none;
}

label *,
label::after,
label::before {
    box-sizing: inherit;
}

c-label {
    position: relative;
}

.check-icon {
    position: relative;
    border-style: var(--forms-border-style);
    border-color: var(--border-default-color);
    border-width: var(--forms-border-width-base);
    border-radius: var(--forms-border-radius);
    width: var(--checkbox-size);
    height: var(--checkbox-size);
    margin-right: 0.5rem;
    align-self: center;
    transition: background-color var(--transition-duration) var(--transition-timing-function);
}

.check-icon::after {
    display: none;
    content: "";
    width: calc(var(--check-size) / 1.5);
    height: calc(var(--check-size) / 3);
    border-bottom: 2px solid var(--color-c-checkbox-arrow);
    border-left: 2px solid var(--color-c-checkbox-arrow);
    transform: rotate(-45deg);
    margin: 30% auto auto;
}

.check-text {
    width: 50%;
    flex-grow: 1;
    display: flex;
    flex-direction: column;
    align-self: center;
}

/* -------------------- VARIATION -- label-left -------------------- */
:host([label-position="left"]) .multioptions {
    display: flex;
    flex-wrap: wrap;
    align-items: baseline;
}

:host([label-position="left"]) .content-wrapper {
    flex-grow: 1000;
    flex-basis: var(--forms-label-left-input-size);
}

:host([label-position="left"]) .label-text {
    flex-basis: var(--forms-label-left-label-size);
    margin-right: var(--forms-label-left-label-spacing);
}

/* -------------------- VARIATION -- type button -------------------- */
:host([button]) {
    --forms-color-disabled: var(--color-neutral-60);
}

:host([button]) .content-wrapper {
    display: flex;
    flex-wrap: wrap;
    gap: 0 0.25em;
}

:host([button]) .input-wrapper {
    background: var(--forms-background);
    border-radius: 1em;
    padding: 0 1em 0 0.5em;
}

:host([button]) .check-icon {
    border-radius: 100%;
}

/* -------------------- STATES -------------------- */

/* Checked */
/* stylelint-disable no-descending-specificity */
input:checked + label .check-icon {
    border-width: 1px;
    border-color: var(--color-interactive-area);
    background-color: var(--color-interactive-area);
}

input:checked + label .check-icon::after {
    display: block;
}

input:focus + label .check-icon,
input:checked:focus + label .check-icon {
    outline: var(--forms-outline);
    outline-offset: var(--forms-outline-offset-checks);
}

/* Hover */
input:hover:not(:disabled) + label .check-icon {
    border-color: var(--forms-border-color-hover);
    background-color: var(--forms-border-color-hover);
}

/* Error */
:host([error]) c-label,
.error {
    color: var(--color-status-error);
}

:host([error]) c-label::after {
    content: "!";
    font-weight: 900;
    margin-left: 0.25rem;
}

/* Disabled */
input:disabled + label,
input:disabled + label .check-icon {
    color: var(--forms-color-disabled);
    cursor: not-allowed;
}

input:checked:disabled + label .check-icon {
    background-color: var(--forms-color-disabled);
    border-color: var(--forms-color-disabled);
}
/* stylelint-enable no-descending-specificity */`
/* eslint-enable max-len */

/*
 * Define web component tag for c-checkbox.
 */

window.customElements.define('c-checkbox', CCheckboxComponent)

/*
 * Default export the concrete class for c-checkbox.
 */

export default CCheckboxComponent
