/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CFormComponent extends BaseComponent {

    /**
     * Create a form field and return it.
     *
     * @param   {String} name               The name of the field
     * @param   {String|FileList} value     The value of the field
     *
     * @return  {HTMLInputElement}          The HTML input field
     */

    static createFormField(name, value) {
        const field = document.createElement('input')

        field.name  = name

        if (value instanceof FileList) {
            field.type   = 'file'
            field.files  = value
            field.hidden = true
        }
        else {
            field.type  = 'hidden'
            field.value = value
        }

        return field
    }

    /**
     * Map the transmitted data with the already existing data for the
     * form component.
     */

    connectedCallback() {
        super.connectedCallback()

        /*
         * Initialise the form data and add event listeners to update the form
         * data in two cases:
         *
         *      1. When the form components update their data & send the
         *         ``updateFormData`` event.
         *
         *      2. When the slot of the this form component changes.
         */

        this.formData = new FormData()

        this.addEventListener('updateFormData', event => {
            this.updateFormData(event.info.component.formData)
        })

        this.shadowRoot.addEventListener('slotchange', event => {
            for (const element of event.target.assignedElements())
                this.updateFormData(element.formData)
        })

        /*
         * Submit the form when the submit button is pressed and sends the
         * ``submitForm`` event.
         */

        this.addEventListener('submitForm', this.submit)
    }

    /**
     * Update the ``formData`` property of this component with the new form
     * data.
     *
     * @param   {Object} formData       The new form data
     */

    updateFormData(formData) {
        if (formData) {
            this.logger.debug('   🔤 Update form data', formData)
            this.formData = {...this.formData, ...formData}
        }
    }

    /**
     * Submit the form.
     */

    submit() {
        const form = this.shadowRoot.querySelector('form')

        /* eslint-disable no-unused-vars */

        // Get valid form data entries (filter undefined values).
        const entries = Object
            .entries(this.formData)
            .filter(([name, value]) => typeof value !== 'undefined')

        // Add fields with single values to the form.
        entries
            .filter(([name, value]) => !Array.isArray(value))
            .forEach(([name, value]) => {
                form.appendChild(CFormComponent.createFormField(name, value))
            })

        // Add fields with multiple values to the form.
        entries
            .filter(([name, value]) => Array.isArray(value))
            .forEach(([name, values]) => {
                for (const value of values)
                    form.appendChild(CFormComponent.createFormField(name, value))
            })

        /* eslint-enable no-unused-vars */

        if (form.querySelector('input[type=file]'))
            form.enctype = 'multipart/form-data'

        form.submit()
    }

}

/*
 * HTML template & CSS style for c-form.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CFormComponent.template = `<form method="\${attributes.method || 'POST'}" action="\${attributes.action || ''}">
</form>
<slot></slot>`


/* eslint-enable max-len */

/*
 * Define web component tag for c-form.
 */

window.customElements.define('c-form', CFormComponent)

/*
 * Default export the concrete class for c-form.
 */

export default CFormComponent
