/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CNavigationComponent extends BaseComponent {

    constructor() {
        super()

        /*
         * Callback for clicks on the document, to close the filter in case the
         * user clicks outside of the component.
         */

        const clickCallback = event => {
            if (event.target.closest('c-select') !== this && this.isFilterOpen) {
                this.logger.debug('Closing filter by clicking outside of component')
                this.closeFilter()
            }
        }

        /*
         * Functions / methods to open and close the filter modal.
         *
         * WARNING: This has to be defined here instead of a normal function,
         * because some JavaScript / ES6 magic will only add the event listener
         * once for all instances when a normal function is used. However, we
         * need separate event listeners for each instance.
         */

        this.openFilter = toggleNode => {
            this.logger.debug('Opening filter')
            toggleNode.toggleAttribute('open')
            document.addEventListener('click', clickCallback)
            this.setAttribute('open', '')
        }

        this.closeFilter = toggleNode => {
            this.logger.debug('Closing filter')
            toggleNode.removeAttribute('open')
            document.removeEventListener('click', clickCallback)
            this.removeAttribute('open')
        }
    }

    /**
     * Check if the filter is currently open.
     *
     * @return  {Boolean}   Is the filter open
     */

    isFilterOpen(toggleNode) {
        return toggleNode.hasAttribute('open')
    }

    /**
     * Toggle the filter.
     */

    toggleSubnavigation(event) {
        const [toggleButton] = event.composedPath()
        const toggleClicked = toggleButton.parentNode
        const toggles = this.shadowRoot.querySelectorAll('.toggle')

        this.logger.debug('Toggle filter')

        toggles.forEach(toggleNode => {
            if (toggleNode === toggleClicked)
                this.openFilter(toggleNode)
            else
                this.closeFilter(toggleNode)
        })
    }

    /**
     * Set the HTML boolean attributes on the form input fields
     */

    render() {
        super.render()

        const toggleButtons = this.shadowRoot.querySelectorAll('.toggle-button')

        toggleButtons.forEach(button => {
            button.addEventListener('click', event => this.toggleSubnavigation(event))
        })
    }

}

/*
 * HTML template & CSS style for c-navigation.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CNavigationComponent.template = `<nav>
    <slot></slot>

    <ul t-if="data.items" class="level-1">
        <li t-for="data.items">
            <span class="toggle" id="toggle" open>
                <c-link
                    href="\${item.href || '#'}"
                    title="\${item.title || ''}"
                    target="\${item.target || '_self'}"
                    state="\${item.state || ''}"
                    class="\${item.class || ''} \${item.status || ''}"
                >\${item.name}
                </c-link>
                <!--<c-button t-on="click:toggle" t-if="item.items" icon="chevron-down" padded tertiary class="toggle"></c-button>-->
                <span class="toggle-button">
                    <c-icon t-if="item.items" icon="chevron-down"></c-icon>
                </span>
            </span>
            <ul t-if="item.items" class="level-2">
                <li t-for="item.items">
                    <c-link
                        href="\${item.href || '#'}"
                        title="\${item.title || ''}"
                        target="\${item.target || '_self'}"
                        state="\${item.state || ''}"
                        class="\${item.class || ''} \${item.status || ''}"
                    >\${item.name}
                    </c-link>
                </li>
            </ul>
        </li>
    </ul>
</nav>`

CNavigationComponent.style = `:host {
    --link-border: none;
    --text-transform: none;
    --c-link-padding: 0.5em 0;
    --c-link-letter-spacing: 0.03em;
    --c-link-font-weight: 500;

    list-style: none;
    margin: 0;
    padding: 0;
    text-transform: var(--text-transform);
    display: inline-block;
    width: 100%;
}

c-link,
::slotted(c-link) {
    --letter-spacing: var(--c-link-letter-spacing);
    --color-interactive-link-border: transparent;
    --font-weight: var(--c-link-font-weight);
    --color-interactive-hover: inherit;
    --link-padding: var(--c-link-padding);

    display: block;
}

.toggle-button {
    --icon-color: var(--border-default-color);

    padding: 0.5em;
    align-self: center;
    transition: var(--transition-property) var(--transition-duration) var(--transition-timing-function);
}

.toggle-button c-icon {
    pointer-events: none;
}

[open] .toggle-button {
    transform: rotate(180deg);
}

.toggle-button:hover {
    --icon-color: inherit;

    cursor: pointer;
}

ul {
    list-style: none;
    margin: 0;
    padding: 0;
}

li {
    border-top: var(--link-border);
}

.toggle {
    display: flex;
}

/* -------------------- VARIATION -- Uppercase ------------------------- */
:host([uppercase]) {
    --c-link-letter-spacing: 0.15em;
    --c-link-font-weight: 300;

    text-transform: uppercase;
}

/* -------------------- Multilevel Nav --------------------------------- */
.level-2 {
    --link-border: none;
    --transition-duration: 500ms;

    overflow: hidden;
    background: var(--color-ui-1);
    opacity: 0;
    max-height: 0;
    transition: var(--transition-property) var(--transition-duration) var(--transition-timing-function);
    box-shadow: var(--box-shadow-default);
    border-radius: 5px;
}

[open] ~ .level-2 {
    opacity: 1;
    max-height: 300px; /* Use a fixed px value for the transition effect */
}

:host([horizontal]) .level-2 {
    --level-inset: 0;

    position: absolute;
}

.level-2 c-link {
    display: flex;
}

.level-2 li {
    margin: 0 2rem;
}

.level-2 li:first-of-type {
    margin-top: 0.5rem;
}

.level-2 li:last-of-type {
    margin-bottom: 0.5rem;
}

/* -------------------- VARIATION -- vertical Nav ---------------------- */
:host(:not([horizontal])) {
    --border-default-width: 1px;
    --link-border: var(--border-default);
}

/* -------------------- VARIATION -- horizontal Nav -------------------- */
:host([horizontal]) {
    --level-inset: 0;
}

:host([horizontal]) nav,
:host([horizontal]) div,
:host([horizontal]) .level-1 {
    display: flex;
    flex-wrap: wrap;
    gap: 1.5rem;
}`
/* eslint-enable max-len */

/*
 * Define web component tag for c-navigation.
 */

window.customElements.define('c-navigation', CNavigationComponent)

/*
 * Default export the concrete class for c-navigation.
 */

export default CNavigationComponent
