/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CRadioComponent extends BaseComponent {

    /**
     * The checked items, returned in a key-value object, where the object key
     * is the data value, and the object value is the data label.
     *
     * @return  {Object}    The checked items
     */

    get checked() {
        const checked = this.shadowRoot.querySelectorAll('input:checked:not(:disabled)')
        return [...checked].reduce((obj, el) => {
            obj[el.value] = el.dataset.label
            return obj
        }, {})
    }

    /**
     * The data of this component, which should be transferred when a form is
     * submitted.
     *
     * @return  {Object}    The form data
     */

    get formData() {
        return {
            [this.componentAttributes.name]: Object.keys(this.checked)[0],
        }
    }

    /**
     * The custom context, which is used in the `template.html` file of the
     * c-radio component.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        const data = this.data ? {...this.data} : ''
        const attributes = {...this.componentAttributes, ...this.data}

        let state = ''
        state += attributes.error ? 'error ' : ''
        state += attributes.disabled ? 'disabled ' : ''
        state += attributes.required ? 'required ' : ''

        return {
            data,
            state,
        }
    }

    /**
     * Reflect the input attributes after rendering.
     */

    render() {
        super.render()

        this.reflectComponentAttributes(
            [
                'checked',
                'disabled',
            ],
            this.shadowRoot.querySelectorAll('input'),
        )

        this.reflectDataProperties(
            [
                'checked',
                'disabled',
            ],
            this.shadowRoot.querySelectorAll('.input-wrapper'),
            element => element.querySelector('input'),
        )
    }

}

/*
 * HTML template & CSS style for c-radio.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CRadioComponent.template = `<fieldset class="\${state}" t-if="data">
    <div t-if="data">
        <c-label t-if="attributes.label" class="label-text \${state}" for="\${name}" required="\${attributes.required}">
            \${attributes.label}
        </c-label>
    </div>

    <div class="content-wrapper">
        <span t-for="data" class="input-wrapper">
            <input
                type="radio"
                id="\${item.value.value}"
                name="\${attributes.name}"
                value="\${item.value.value}"
                data-label="\${item.value.label}"
                t-on="change:updateFormData"
            >

            <label for="\${item.value.value}">
                <span class="radio-button"></span>
                \${item.value.label}
            </label>
        </span>

        <div t-if="attributes.help" class="help">
                \${attributes.help}
            </div>
            <slot name="help" class="help"></slot>

        <div t-if="attributes.error" class="error-text">
            \${attributes.error}
        </div>
    </div>
</fieldset>`

CRadioComponent.style = `/* ============================================================ *
   GENERIC
 * ============================================================ */

/* -------------------- RESETS -------------------- */
*,
::before,
::after {
    box-sizing: border-box;
}

/*
 * Change the cursor on disabled, not-editable, or otherwise
 * inoperable elements in all browsers (opinionated).
 */

[aria-disabled="true"],
[disabled] {
    cursor: not-allowed;
}

/* ============================================================ *
   ELEMENTS
 * ============================================================ */

:host {
    --radio-button-size: 1rem;
    --radio-background: var(--color-interactive-area);

    display: block;
    width: 100%;
    margin-bottom: var(--forms-margin);
}

fieldset {
    padding: 0;
    border: 0;
}

input {
    position: absolute;
    overflow: hidden;
    width: 1px;
    height: 1px;
    padding: 0;
    border: 0;
    margin: -1px;
    clip: rect(0, 0, 0, 0);
    white-space: nowrap;
    visibility: inherit;
}

.error-text {
    font-size: var(--forms-error-font-size);
}

.help {
    font-size: var(--forms-help-font-size);
    color: var(--forms-color-help);
    font-style: var(--forms-help-font-style);
    display: inline;
    width: 100%;
}

label {
    display: flex;
    align-items: center;
    justify-content: left;
    cursor: pointer;
}

label *,
label::after,
label::before {
    box-sizing: inherit;
}

c-label {
    position: relative;
}

.radio-button {
    display: flex;
    align-items: center;
    justify-content: center;
    box-sizing: border-box;
    padding: 0;
    width: var(--radio-button-size);
    height: var(--radio-button-size);
    flex-shrink: 0;
    border: var(--border-default);
    margin-right: 0.5rem;
    background-color: transparent;
    border-radius: 50%;
}

.radio-button::before {
    content: "";
    position: relative;
    display: inline-block;
    width: calc(var(--radio-button-size) / 2);
    height: calc(var(--radio-button-size) / 2);
    border-radius: 50%;
    background-color: transparent;
}

/* -------------------- VARIATION -- label-left -------------------- */
:host([label-position="left"]) fieldset,
:host([label-position="left"]) c-label {
    display: flex;
    flex-wrap: wrap;
    align-items: baseline;
}

:host([label-position="left"]) .content-wrapper {
    flex-grow: 1000;
    flex-basis: var(--forms-label-left-input-size);
}

:host([label-position="left"]) .label-text {
    flex-basis: var(--forms-label-left-label-size);
    margin-right: var(--forms-label-left-label-spacing);
}

/* -------------------- VARIATION -- type button -------------------- */
:host([button]) {
    --forms-color-disabled: var(--color-neutral-60);
    --checkbox-size: 1rem;
    --check-size: 90%;
}

:host([button]) .content-wrapper {
    display: flex;
    flex-wrap: wrap;
    gap: 0 0.25em;
}

:host([button]) .input-wrapper {
    background: var(--forms-background);
    border-radius: 1em;
    padding: 0 1em 0 0.5em;
}

:host([button]) .radio-button {
    position: relative;
    border-style: var(--forms-border-style);
    border-color: var(--border-default-color);
    border-width: var(--forms-border-width-base);
    width: var(--checkbox-size);
    height: var(--checkbox-size);
    margin-right: 0.5rem;
    align-self: center;
    transition: background-color var(--transition-duration) var(--transition-timing-function);
}

:host([button]) .radio-button::before {
    display: none;
    border-radius: 100%;
}

:host([button]) .radio-button::after {
    display: none;
    content: "";
    width: calc(var(--check-size) / 1.5);
    height: calc(var(--check-size) / 3);
    border-bottom: 2px solid var(--color-c-checkbox-arrow);
    border-left: 2px solid var(--color-c-checkbox-arrow);
    transform: rotate(-45deg);
    margin: 30% auto auto;
}

:host([button]) input:checked + label .radio-button {
    border-width: 1px;
    border-color: var(--color-interactive-area);
    background-color: var(--color-interactive-area);
}

:host([button]) input:checked + label .radio-button::after {
    display: block;
}

:host([button]) input:focus + label .radio-button,
input:checked:focus + label .check-icon {
    outline: var(--forms-outline);
    outline-offset: var(--forms-outline-offset-checks);
}

/* Hover */
:host([button]) input:hover:not(:disabled) + label .radio-button {
    border-color: var(--forms-border-color-hover);
    background-color: var(--forms-border-color-hover);
}

/* -------------------- STATES -------------------- */
/* stylelint-disable no-descending-specificity */

/* Checked */
input:checked + label .radio-button::before {
    background-color: var(--radio-background);
}

input:focus + label .radio-button,
input:checked:focus + label .radio-button {
    outline: 1px solid transparent;
    box-shadow: 0 0 0 var(--forms-outline-size) var(--forms-outline-color);
}

/* Hover */
input:hover:not(:disabled) + label .radio-button::before {
    background-color: var(--forms-border-color-hover);
}

/* Error */
.error-text,
.error c-label,
.error c-label::after {
    color: var(--color-status-error);
}
/* stylelint-enable no-descending-specificity */

.error c-label::after {
    content: "!";
    font-weight: 900;
    margin-left: 0.25rem;
}

/* Disabled */
input:disabled + label {
    color: var(--forms-color-disabled);
    cursor: not-allowed;
}

/* stylelint-disable no-descending-specificity */
input:disabled + label .radio-button {
    border-color: var(--forms-color-disabled);
}

input:disabled:checked + label .radio-button::before {
    background-color: var(--forms-color-disabled);
}
/* stylelint-enable no-descending-specificity */`
/* eslint-enable max-len */

/*
 * Define web component tag for c-radio.
 */

window.customElements.define('c-radio', CRadioComponent)

/*
 * Default export the concrete class for c-radio.
 */

export default CRadioComponent
