/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CSelectComponent extends BaseComponent {

    constructor() {
        super()

        /*
         * Callback for clicks on the document, to close the filter in case the
         * user clicks outside of the component.
         */

        const clickCallback = event => {
            if (event.target.closest('c-select') !== this && this.isFilterOpen) {
                this.logger.debug('Closing filter by clicking outside of component')
                this.closeFilter()
            }
        }

        /*
         * Functions / methods to open and close the filter modal.
         *
         * WARNING: This has to be defined here instead of a normal function,
         * because some JavaScript / ES6 magic will only add the event listener
         * once for all instances when a normal function is used. However, we
         * need separate event listeners for each instance.
         */

        this.openFilter = () => {
            this.logger.debug('Opening filter')
            document.addEventListener('click', clickCallback)
            this.setAttribute('open', '')
        }

        this.closeFilter = () => {
            this.logger.debug('Closing filter')
            document.removeEventListener('click', clickCallback)
            this.removeAttribute('open')
        }
    }

    /**
     * The data of this component, which should be transferred when a form is
     * submitted.
     *
     * @return  {Object}    The form data
     */

    get formData() {
        return this.shadowRoot.querySelector('#input').formData
    }

    /**
     * The custom context, which is used in the `template.html` file of the
     * c-select component.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        const data = {...this.data}
        const attributes = {...this.componentAttributes}
        const labelInside = data && attributes['label-position'] === 'inside'
        const labelPosition = attributes['label-position']

        return {
            data,
            labelInside,
            labelPosition,
        }
    }

    /**
     * Check if the filter is currently open.
     *
     * @return  {Boolean}   Is the filter open
     */

    get isFilterOpen() {
        return this.hasAttribute('open')
    }

    /**
     * Toggle the filter.
     */

    toggleFilter() {
        this.logger.debug('Toggle filter')
        if (this.isFilterOpen)
            this.closeFilter()
        else
            this.openFilter()
    }

    /**
     * Update the placeholder.
     */

    updatePlaceholder() {
        const values = Object.values(this.shadowRoot.getElementById('input').checked)
        if (values && values[0])
            this.shadowRoot.getElementById('placeholder').textContent = values.join(', ')
    }

    /**
     * Set the HTML boolean attributes on the form input fields
     */

    render() {
        super.render()
        this.updatePlaceholder()

        this.reflectComponentAttributes(
            ['disabled'],
            this.shadowRoot.querySelectorAll('button'),
        )

        this.shadowRoot.getElementById('filter-toggle-button')
            .addEventListener('click', () => this.toggleFilter())

        this.shadowRoot.getElementById('input')
            .addEventListener('updateFormData', () => this.updatePlaceholder())

        this.shadowRoot.querySelector('c-radio')
            .addEventListener('updateFormData', () => this.toggleFilter())
    }

}

/*
 * HTML template & CSS style for c-select.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CSelectComponent.template = `<label>

    <c-label t-if="attributes.label && !labelInside"
             required="\${attributes.required}"
             label-position="\${labelPosition}"
             class="label-text">
        \${attributes.label}
    </c-label>

    <div id="filter-state" class="content-wrapper">
        <div class="input-wrapper">

            <c-label t-if="attributes.label && labelInside"
                     required="\${attributes.required}"
                     label-position="\${labelPosition}"
                     class="label-text">
                \${attributes.label}
            </c-label>

            <div class="select-wrapper">
                <button id="filter-toggle-button" class="filter-toggle">
                    <div id="placeholder" class="filter-toggle-text">\${attributes.placeholder || 'Please Select an option'}</div>
                    <div class="filter-toggle-icon"></div>
                </button>
            </div>
        </div>

        <div t-if="attributes.help" class="help">
            \${attributes.help}
        </div>
        <slot name="help" class="help"></slot>

        <div t-if="attributes.error" class="error">
            \${attributes.error}
        </div>

        <div class="filter-modal">
            <div class="filter-modal-wrapper">
                <div class="filter-modal-header">
                    <c-input t-if="attributes.hassearch" placeholder="Filter&hellip;"></c-input>
                    <c-button id="filter-close-button" icon="cross" secondary></c-button>
                </div>
                <div t-if="attributes.multiple">
                    <c-checkbox id="input" name="\${attributes.name}" data-data="\${data ? JSON.stringify(data, null, 4) : ''}"></c-checkbox>
                </div>
                <div t-if="!attributes.multiple">
                    <c-radio id="input" name="\${attributes.name}" data-data="\${data ? JSON.stringify(data, null, 4) : ''}"></c-radio>
                </div>
            </div>
        </div>
    </div>

</label>`

CSelectComponent.style = `/* ============================================================ *
   GENERIC
 * ============================================================ */

/* -------------------- RESETS -------------------- */
*,
::before,
::after {
    box-sizing: border-box;
}

/*
 * Change the cursor on disabled, not-editable, or otherwise
 * inoperable elements in all browsers (opinionated).
 */

[aria-disabled="true"],
[disabled] {
    cursor: not-allowed;
}

/* ============================================================ *
   ELEMENTS
 * ============================================================ */

:host {
    --select-arrow-size: 1rem;
    --select-arrow-icon-spacing: 2.75rem;
    --select-arrow-text-spacing: 2rem;
    --select-modal-width: 20rem;
    --select-modal-height: 12rem;
    --select-modal-spacing: 0.5rem;

    display: block;
    width: 100%;
    margin-bottom: 1rem;
}

label {
    display: block;
}

.content-wrapper {
    display: flex;
    flex-flow: column;
    position: relative;
}

.input-wrapper {
    position: relative;
    width: 100%;
    display: flex;
    flex-wrap: wrap;
    min-height: var(--forms-height);
    border-style: var(--forms-border-style);
    border-color: var(--forms-border-color);
    border-width: var(--forms-border-width);
    border-radius: var(--forms-border-radius);
    margin-top: var(--forms-border-shift);
    outline: 0;
    background: var(--forms-background);
    color: inherit;
    padding: var(--forms-padding);
}

.select-wrapper {
    position: relative;
    width: 100%;
}

.filter-toggle {
    width: 100%;
    display: inline-flex;
    justify-content: space-between;
    padding: 0;
    background: transparent;
    font-size: inherit;
    font-family: inherit;
    outline: 0;
    border: 0;
    color: inherit;
    text-align: left;
}

.filter-toggle-text {
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    width: calc(100% - var(--select-arrow-text-spacing));
}

.filter-toggle-icon::before {
    content: "";
    position: absolute;
    right: 0.5rem;
    width: var(--select-arrow-size);
    height: var(--select-arrow-size);
    border-bottom: 1px solid var(--color-interactive-button-text);
    border-left: 1px solid var(--color-interactive-button-text);
    top: 50%;
    transform: translateY(-75%) rotate(-45deg);
}

[disabled] .filter-toggle-icon::before {
    border-color: var(--forms-color-disabled);
}

:host([open]) .filter-toggle-icon::before {
    transform: translateY(-25%) rotate(135deg);
}

.filter-modal {
    display: none;
    position: absolute;
    left: 0;
    top: 2.25rem;
    background: var(--color-ui-bg);
    padding: var(--select-modal-spacing);
    box-shadow: var(--box-shadow-default);
    width: var(--select-modal-width);
    z-index: var(--z-index-front);
    border: var(--border-default-width) solid var(--border-color-c-select-modal);
}

:host([open]) .filter-modal {
    display: block;
}

.filter-modal-wrapper {
    height: var(--select-modal-height);
    padding: var(--select-modal-spacing);
    overflow: auto;
}

.filter-modal-header {
    display: flex;
    margin-bottom: 1rem;
}

.filter-modal-header c-input {
    margin-right: 1.5rem;
    margin-bottom: 0;
}

.filter-modal-header c-button {
    margin-left: auto;
}

:host([error]) {
    --select-arrow-text-spacing: 3rem;
}

.error {
    font-size: var(--forms-error-font-size);
    padding: 0 var(--forms-padding-horizontal);
}

.help {
    font-size: var(--forms-help-font-size);
    color: var(--forms-color-help);
    font-style: var(--forms-help-font-style);
    padding: 0 var(--forms-padding-horizontal);
    display: inline;
}

/* -------------------- VARIATION -- label-left -------------------- */
:host([label-position="left"]) label,
:host([label-position="left"]) c-label {
    display: flex;
    flex-wrap: wrap;
    align-items: baseline;
    gap: 0 var(--forms-label-left-label-spacing);
}

:host([label-position="left"]) .content-wrapper {
    flex-grow: 1000;
    flex-basis: var(--forms-label-left-input-size);
    width: 0; /* Ensure a single line label-select combination */
}

:host([label-position="left"]) .label-text {
    flex-basis: var(--forms-label-left-label-size);
}

/* -------------------- VARIATION -- label-inside -------------------- */
:host([label-position="inside"]) .input-wrapper {
    display: flex;
    align-items: baseline;
}

:host([label-position="inside"]) .label-text {
    display: flex;
    flex-basis: var(--forms-label-left-label-size);
}

:host([label-position="inside"]) .select-wrapper {
    width: auto;
    flex-grow: 1;
}

/* -------------------- STATES -------------------- */
/* stylelint-disable no-descending-specificity */
.input-wrapper:hover {
    border-color: var(--forms-border-color-hover);
}

.input-wrapper:focus-within {
    outline: var(--forms-outline);
    outline-offset: var(--forms-outline-offset);
}
/* stylelint-enable no-descending-specificity */

/* Error */
.error,
:host([error]) .label-text,
:host([error]) .input-wrapper::after {
    color: var(--color-status-error);
}

:host([error]) .input-wrapper {
    border-color: var(--color-status-error);
}

:host([error]) .input-wrapper::after {
    content: "!";
    position: absolute;
    right: var(--select-arrow-icon-spacing);
    font-weight: 900;
    top: 50%;
    transform: translateY(-50%);
}

:host([error]) select {
    padding-right: calc(var(--select-arrow-icon-spacing) + 0.5rem);
}

/* Disabled */
:host([disabled]) {
    color: var(--forms-color-disabled);
}

:host([disabled]) .input-wrapper {
    border-color: var(--forms-border-color-disabled);
    background: var(--forms-background-disabled);
}

:host([disabled]) .input-wrapper:hover {
    border-color: var(--forms-border-color-disabled);
}`
/* eslint-enable max-len */

/*
 * Define web component tag for c-select.
 */

window.customElements.define('c-select', CSelectComponent)

/*
 * Default export the concrete class for c-select.
 */

export default CSelectComponent
