/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

import ComponentEvent from '../event.js?v=1.12.1'

const SELECTED_SELECTOR = '#selected'
const UNSELECTED_SELECTOR = '#unselected'

class CSplitSelectComponent extends BaseComponent {

    /**
     * The custom context, which is used in the `template.html` file of the
     * c-select component.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        const data = {...this.data}
        const attributes = {...this.componentAttributes}
        const labelPosition = attributes['label-position']

        return {
            data,
            labelPosition,
        }
    }

    /**
     * The data of this component, which should be transferred when a form is
     * submitted.
     *
     * @return  {Object}    The form data
     */

    get formData() {
        const options = this.shadowRoot.querySelector(SELECTED_SELECTOR)
        return {
            [this.componentAttributes.name]: Array.from(options).map(option => option.value),
        }
    }

    /**
     * Clear the options in the select boxes.
     */

    clearSelects() {
        for (const select of this.shadowRoot.querySelectorAll('select'))
            select.parentNode.replaceChild(select.cloneNode(false), select)
    }

    /**
     * Populate the select boxes with the options, based on the input data.
     */

    populateSelects() {
        for (const item of this.data) {
            const selector = item.checked ? SELECTED_SELECTOR : UNSELECTED_SELECTOR
            const select   = this.shadowRoot.querySelector(selector)
            const option   = document.createElement('option')

            option.value     = item.value
            option.textContent = item.label
            select.appendChild(option)
        }
    }

    /**
     * Find the next sibling option, for a specific value in a specific select.
     *
     * @param   {HTMLElement} select    The select element
     * @param   {String} value          The value which should be inserted
     *
     * @return  {HTMLElement}           The next sibling option
     */

    findNextSiblingForValue(select, value) {
        const values         = this.data.map(item => item.value)
        const eligableValues = values.filter(item => values.indexOf(item) > values.indexOf(value))

        for (const eligableValue of eligableValues) {
            const option = select.querySelector(`option[value="${eligableValue}"]`)
            if (option)
                return option
        }

        return null
    }

    /**
     * Move options from the source select to the target select.
     *
     * @param   {String} sourceSelector     The selector for the source select
     * @param   {String} targetSelector     The selector for the target select
     */

    moveOptions(sourceSelector, targetSelector) {
        const targetSelect = this.shadowRoot.querySelector(targetSelector)
        for (const option of this.shadowRoot.querySelectorAll(`${sourceSelector} option:checked`)) {
            option.selected = false

            const nextOption = this.findNextSiblingForValue(targetSelect, option.value)
            if (nextOption)
                targetSelect.insertBefore(option, nextOption)
            else
                targetSelect.appendChild(option)
        }

        this.dispatchEvent(new ComponentEvent('updateFormData'))
    }

    /**
     * Move the selected options from the "unselected select" into the "selected
     * select" box. In other terms, move the selected options on the left to
     * right.
     */

    selectOptions() {
        this.moveOptions(UNSELECTED_SELECTOR, SELECTED_SELECTOR)
    }

    /**
     * Move the selected options from the "selected select" into the "unselected
     * select" box. In other terms, move the selected options on the right to
     * left.
     */

    unselectOptions() {
        this.moveOptions(SELECTED_SELECTOR, UNSELECTED_SELECTOR)
    }

    /**
     * Filter select options.
     *
     * @param   {String} filter     The filter term
     */

    filterOptions(filter) {
        try {
            const safeFilter = filter.replace(/[^a-zA-Z0-9\s]/gu, '')

            if (safeFilter.length > 0) {
                // eslint-disable-next-line max-len
                for (const option of this.shadowRoot.querySelectorAll(`${UNSELECTED_SELECTOR} option`))
                    option.hidden = !option.textContent.includes(safeFilter)
            }
            else
                // eslint-disable-next-line max-len
                this.logger.warn('The filter is empty or contains dangerous characters, skipping it: ', filter)
        }
            // eslint-disable-next-line no-unused-vars
            catch (ex) {
            this.logger.warn('Ignoring filter pattern because it\'s not valid: ', filter)
        }
    }



    /**
     * Render the component.
     */

    render() {
        super.render()

        this.clearSelects()
        this.populateSelects()

        this.shadowRoot.querySelector('#filter').addEventListener('keyup', event => {
            this.filterOptions(event.target.formData.filter)
        })

        this.shadowRoot.querySelector('#select').addEventListener('click', () => {
            this.selectOptions()
        })

        this.shadowRoot.querySelector('#unselect').addEventListener('click', () => {
            this.unselectOptions()
        })
    }

}

/*
 * HTML template & CSS style for c-split-select.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CSplitSelectComponent.template = `<label>

    <c-label t-if="attributes.label"
             required="\${attributes.required}"
             label-position="\${labelPosition}"
             class="label-text">
        \${attributes.label}
    </c-label>

    <div class="content-wrapper">
        <div class="select-wrapper">
            <div class="selector-available">
                <header>
                    <slot name="header-available">Available Options</slot>
                </header>
                <div class="filter">
                    <c-input id="filter" name="filter" placeholder="RegEx filter…"></c-input>
                </div>

                <select id="unselected" multiple></select>
            </div>

            <div class="selector-choser">
                <c-button id="select" icon="arrow-right" secondary></c-button>
                <c-button id="unselect" icon="arrow-left" secondary></c-button>
            </div>

            <div class="selector-chosen">
                <header>
                    <slot name="header-chosen">Chosen Options</slot>
                </header>
                <select id="selected" multiple></select>
            </div>
        </div>

        <div t-if="attributes.help" class="help">
            \${attributes.help}
        </div>
        <slot name="help" class="help"></slot>

        <div t-if="attributes.error" class="error">
            \${attributes.error}
        </div>
    </div>

</label>`

CSplitSelectComponent.style = `:host {
    --border-color: var(--color-ui-3);

    display: block;
    margin-bottom: 1rem;
}

.content-wrapper {
    display: flex;
    flex-flow: column;
    position: relative;
}

.input-wrapper {
    position: relative;
    width: 100%;
    display: flex;
    flex-wrap: wrap;
    min-height: var(--forms-height);
    border-style: var(--forms-border-style);
    border-color: var(--forms-border-color);
    border-width: var(--forms-border-width);
    border-radius: var(--forms-border-radius);
    margin-top: var(--forms-border-shift);
    outline: 0;
    background: var(--forms-background);
    color: inherit;
    padding: var(--forms-padding);
}

.select-wrapper {
    display: flex;
    width: 100%;
    height: 30vh;
    min-height: 15rem;
}

/* -------------------- CONTAINER -------------- */
.selector-available,
.selector-chosen {
    flex: 1;
    display: flex;
    flex-direction: column;
    border: var(--border-default-width) solid var(--border-default-color);
}

.selector-choser {
    margin: 5rem 0.375rem 0;
    display: flex;
    flex-direction: column;
    justify-content: center;
}

/* -------------------- ELEMENTS -------------- */
header {
    background: var(--color-ui-1);
    font-size: 0.75rem;
    text-transform: uppercase;
    letter-spacing: var(--letter-spacing-uppercase);
    text-align: center;
    padding: 0.125rem;
}

.filter {
    margin: 0.75rem;
}

c-input {
    margin: 0;
}

select {
    width: 100%;
    border: 0;
    border-top: var(--border-default-width) solid var(--border-default-color);
    background: transparent;
    height: 100%;
    padding: 0.5rem;
    color: inherit;
}

select:focus-within {
    outline: var(--forms-outline);
    outline-offset: var(--forms-outline-offset);
}

.selector-choser c-button:not(last-child) {
    margin-bottom: 0.5rem;
}

.error-text {
    font-size: var(--forms-error-font-size);
    padding: 0 var(--forms-padding-horizontal);
}

.help {
    font-size: var(--forms-help-font-size);
    color: var(--forms-color-help);
    font-style: var(--forms-help-font-style);
    padding: 0 var(--forms-padding-horizontal);
    display: inline;
}

/* -------------------- VARIATION -- label-left -------------------- */
:host([label-position="left"]) label,
:host([label-position="left"]) c-label {
    display: flex;
    flex-wrap: wrap;
    align-items: baseline;
    gap: 0 var(--forms-label-left-label-spacing);
}

:host([label-position="left"]) .content-wrapper {
    flex-grow: 1000;
    flex-basis: var(--forms-label-left-input-size);
    width: 0; /* Ensure a single line label-select combination */
}

:host([label-position="left"]) .label-text {
    flex-basis: var(--forms-label-left-label-size);
}

/* -------------------- VARIATION -- label-inside -------------------- */
:host([label-position="inside"]) .input-wrapper {
    display: flex;
    align-items: baseline;
}

:host([label-position="inside"]) .label-text {
    display: flex;
    flex-basis: var(--forms-label-left-label-size);
    width: auto;
}

:host([label-position="inside"]) .select-wrapper {
    width: auto;
    flex-grow: 1;
}

/* -------------------- STATES -------------------- */
/* stylelint-disable no-descending-specificity */
.input-wrapper:hover {
    border-color: var(--forms-border-color-hover);
}

.input-wrapper:focus-within {
    outline: var(--forms-outline);
    outline-offset: var(--forms-outline-offset);
}
/* stylelint-enable no-descending-specificity */

/* Error */
.error,
:host([error]) .label-text,
:host([error]) .input-wrapper::after {
    color: var(--color-status-error);
}

:host([error]) .input-wrapper {
    border-color: var(--color-status-error);
}

:host([error]) .input-wrapper::after {
    content: "!";
    position: absolute;
    right: var(--select-arrow-icon-spacing);
    font-weight: 900;
    top: 50%;
    transform: translateY(-50%);
}

:host([error]) select {
    padding-right: calc(var(--select-arrow-icon-spacing) + 0.5rem);
}

/* Disabled */
:host([disabled]) {
    color: var(--forms-color-disabled);
    cursor: not-allowed;
    pointer-events: none;
}

:host([disabled]) .input-wrapper {
    border-color: var(--forms-border-color-disabled);
    background: var(--forms-background-disabled);
}

:host([disabled]) .input-wrapper:hover {
    border-color: var(--forms-border-color-disabled);
}`
/* eslint-enable max-len */

/*
 * Define web component tag for c-split-select.
 */

window.customElements.define('c-split-select', CSplitSelectComponent)

/*
 * Default export the concrete class for c-split-select.
 */

export default CSplitSelectComponent
