/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CTableComponent extends BaseComponent {

    constructor() {
        super()

        /*
         * Initialise an empty slot context, which will later be populated by
         * the renderTableCell() and injected by the customContext() getter.
         */

        this.slotContext = {}
    }

    /**
     * The custom context, expanded with the slot context.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        return this.slotContext
    }

    /**
     * Get the list of the rendered columns.
     *
     * By default, the columns are automatically determined from the components
     * data. However, the user can overwrite the columns by specifying a
     * columns attribute on the component itself.
     *
     * @return  {Array}     The column names
     */

    getColumns() {
        let columns = []

        if (typeof this.attributes.columns !== 'undefined') {
            for (const column of this.attributes.columns.value.split(';'))
                columns.push(column.trim())
        }
        else if (this.data.length)
            columns = Object.keys(this.data[0]).filter(item => item !== '_meta')

        return columns
    }

    /**
     * Get all slotted elements with a specific attribute on the element itself.
     *
     * The value of the desired attribute is then used as key, while the slotted
     * element itself is used as the value of the returned object.
     *
     * @param   {String} slotAttribute  The name of the attribute
     *
     * @return  {Object}                The slotted elements
     */

    getSlottedElements(slotAttribute) {
        const elements  = {}
        const slot     = this.shadowRoot.querySelector('slot')

        for (const element of slot.assignedElements()) {
            if (element.attributes[slotAttribute])
                elements[element.attributes[slotAttribute].value] = element
        }

        return elements
    }

    /**
     * Render a table cell.
     *
     * @param   {String} tag                The HTML tag used for the cell
     * @param   {String} column             The name of the column
     * @param   {Object} rowData            The data of the row
     * @param   {HTMLElement} customElement The custom element used for the rendering
     *
     * @return  {HTMLElement}               The HTML cell
     */

     
    renderCell(tag, column, rowData, customElement) {
        const cell  = document.createElement(tag)
        const value = rowData[column]

        if (customElement) {
            this.slotContext = rowData
            cell.appendChild(this.renderElement(customElement))
        }
 else if (typeof value === 'boolean') {
            const icon = document.createElement('c-icon')
            icon.setAttribute('icon', value ? 'check' : 'cross')
            icon.setAttribute('status', value ? 'success' : 'error')
            cell.appendChild(icon)
        }
 else 
            cell.textContent = value
        

        return cell
    }

     

    /**
     * Render table rows & cells.
     *
     * @param   {HTMLElement} parentElement     The parent HTML element to assign the row to
     * @param   {String} cellTag                The HTML tag used for the cell
     * @param   {Array} data                    The data array for the rendering of the rows
     * @param   {Object} customElements         An optional mapping of custom column elements
     */

    renderRows(parentElement, cellTag, data, customElements) {
        for (const rowData of data) {
            const row = document.createElement('tr')

            for (const column of this.columns)
                row.appendChild(this.renderCell(cellTag, column, rowData, customElements[column]))

            parentElement.appendChild(row)
        }
    }

    /**
     * Set the top position for the sticky element relative to the header.
     *
     * @param   {HTMLElement} parentElement     The parent HTML element to assign the position to
     */

    setHeaderStickyPosition() {
        const siteHeader = document.querySelector('c-site-header[sticky]')
        const stickyTable = this.parentNode.querySelector('[sticky]')

        if (!stickyTable)
            return

        const tableHeader = stickyTable.shadowRoot.querySelector('thead')

        tableHeader.style.top = `${siteHeader.offsetHeight}px`
    }

    render() {
        super.render()

        this.columns = this.getColumns()

        // Convert ['foo_bar', 'spam_eggs'] to {'foo_bar': 'foo bar', 'spam_eggs': 'spam eggs'}.
        const headers = this.columns.reduce((obj, item) => {
            obj[item] = item.replaceAll('_', ' ')
            return obj
        }, {})

        this.renderRows(
            this.shadowRoot.querySelector('thead'),
            'th',
            [headers],
            this.getSlottedElements('header'),
        )

        this.renderRows(
            this.shadowRoot.querySelector('tbody'),
            'td',
            this.data,
            this.getSlottedElements('column'),
        )

        this.setHeaderStickyPosition()
    }

}

/*
 * HTML template & CSS style for c-table.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CTableComponent.template = `<div class="table-wrapper">
    <table>
        <caption t-if="attributes.caption">\${attributes.caption}</caption>
        <thead></thead>
        <tbody></tbody>
        <slot></slot> <!-- Will automatically be removed by the render() method. -->
    </table>
</div>`

CTableComponent.style = `:host {
    --table-border-width: var(--table-border-width-top) var(--table-border-width-right) var(--table-border-width-bottom) var(--table-border-width-left);
    --table-shadow-height: 0.5rem;

    border-collapse: var(--table-border-collapse);
    display: flex;
    width: 100%;
    overflow: var(--table-overflow, auto);
}

::slotted(*[column]),
::slotted(*[header]) {
    display: none;
}

.table-wrapper {
    position: relative;
    width: var(--table-width, auto);
    max-width: 100%;
}

table {
    border-collapse: separate;
    border-spacing: 0;
    width: 100%;
    word-break: initial;
}

caption {
    text-align: center;
}

thead {
    position: var(--head-position, relative);
    top: 0;
    z-index: var(--z-index-front);
}

@media screen and (width <= 767px) {
    thead {
        top: 0 !important;
    }
}

tbody tr:hover {
    background: var(--color-ui-2);
}

th,
td {
    border-width: var(--table-border-width);
    border-style: var(--table-border-style);
    border-color: var(--table-border-color);
    padding: var(--table-padding);
    vertical-align: var(--table-vertical-alignment);
}

th {
    position: relative;
    background: var(--color-ui-1);
    box-shadow: var(--box-shadow-table-top);
    text-transform: var(--table-head-text-transform);
    letter-spacing: var(--table-head-letter-spacing);
    font-weight: var(--table-head-font-weight);
    border-bottom: var(--table-head-border-bottom);
}

tfoot td {
    border-top: 0.25rem solid var(--border-default-color);
}

:host th:not(:last-child),
:host td:not(:last-child) {
    border-right: 0;
}

:host tr:not(:first-child) td {
    border-top: 0;
}

/* -------------------- TABLE VARIATION -- sticky header -------------------- */
:host([sticky]) {
    --head-position: sticky;
    --table-overflow: none;
}

/* -------------------- TABLE VARIATION -- full width -------------------- */
:host([full-width]) .table-wrapper {
    --table-width: 100%;

    display: table;
}

/* -------------------- TABLE VARIATION -- horizontal rulers -------------------- */
:host([horizontal-rulers]) {
    border: 0;
    margin: 0;

    --table-border-width-left: 0;
    --table-border-width-right: 0;
    --table-border-width-top: 0;
    --table-padding-left-first-column: 0;
    --table-padding-right-last-column: 0;
}

:host([horizontal-rulers]) thead::after {
    --table-padding-top-first-row: calc(2 * var(--table-border-spacing-horizontal));
    --table-padding-left-first-column: 0;

    content: "";
    height: var(--table-padding-top-first-row);
    display: table-row;
}

:host([horizontal-rulers]) tr > *:first-child {
    padding-left: var(--table-padding-left-first-column);
}

:host([horizontal-rulers]) tr > *:last-child {
    padding-right: var(--table-padding-right-last-column);
}

/* -------------------- TABLE VARIATION -- hide last ruler -------------------- */
:host([hide-last-ruler]) tr:last-child td {
    border-bottom-width: 0;
}

/* -------------------- TABLE VARIATION -- spacings -------------------- */

.u-min-width-5 {
    min-width: 5rem;
}

.u-min-width-10 {
    min-width: 10rem;
}`
/* eslint-enable max-len */

/*
 * Define web component tag for c-table.
 */

window.customElements.define('c-table', CTableComponent)

/*
 * Default export the concrete class for c-table.
 */

export default CTableComponent
