/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class CTextareaComponent extends BaseComponent {

    /**
     * The initial or the input form data of the component.
     *
     * @returns     {Object}    The form data
     */

    get formData() {
        return {
            [this.componentAttributes.name]: this.shadowRoot.querySelector('textarea').value,
        }
    }

    /**
     * The custom context, which is used in the `template.html` file of the
     * c-textarea component.
     *
     * @return  {Object}    The custom context
     */

    get customContext() {
        const data = {...this.componentAttributes, ...this.data.inputObject}
        const labelInside = data && data['label-position'] === 'inside'
        const labelPosition = data['label-position']

        let state =  ''
        state += data.error ? 'error ' : ''
        state += data.disabled ? 'disabled ' : ''
        state += data.required ? 'required ' : ''

        return {
            data,
            labelInside,
            labelPosition,
            state,
        }
    }

    /**
     * Reflect the "checked" & "disabled" properties of the data as attributes
     * to the input elements.
     *
     * This is fancy talk for "copy-paste" JSON properties to <input>
     * attributes.
     */

    reflectInputAttributes() {
        const inputElements = this.shadowRoot.querySelectorAll('.input-wrapper')
        const attributes    = ['checked', 'disabled']

        if (inputElements) {
            for (const inputElement of inputElements) {
                const input = inputElement.querySelector('input')

                for (const attribute of attributes) {
                    if (inputElement.data && inputElement.data.value[attribute] === true)
                        input.setAttribute(attribute, attribute)
                }
            }
        }
    }

    /**
     * Reflect the input attributes after rendering.
     */

    render() {
        super.render()

        this.reflectComponentAttributes(
            [
                'disabled',
                'name',
                'placeholder',
                'rows',
                'title',
                'value',
            ],
            this.shadowRoot.querySelectorAll('textarea'),
            element => element,
        )
    }

}

/*
 * HTML template & CSS style for c-textarea.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
CTextareaComponent.template = `<label class="\${state}">
    <c-label t-if="data.label && !labelInside"
             label-position="\${labelPosition}"
             required="\${attributes.required}"
             class="label-text">
        \${data.label}
    </c-label>

    <div class="content-wrapper">
        <div class="input-wrapper">
            <c-label t-if="data.label && labelInside"
                     label-position="\${labelPosition}"
                     required="\${attributes.required}"
                     class="label-text">
                \${data.label}
            </c-label>
            <textarea type="text"
                      placeholder="\${data.placeholder || ''}"
                      rows="2"
                      t-on="change:updateFormData"
            >\${data.value || ''}</textarea>
        </div>

        <div t-if="attributes.help" class="help">
            \${attributes.help}
        </div>
        <slot name="help" class="help"></slot>

        <div t-if="data.error" class="error-text">
            \${data.error}
        </div>
    </div>
</label>`

CTextareaComponent.style = `/* ============================================================ *
   GENERIC
 * ============================================================ */

/* -------------------- RESETS -------------------- */
*,
::before,
::after {
    box-sizing: border-box;
}

/*
 * Change the cursor on disabled, not-editable, or otherwise
 * inoperable elements in all browsers (opinionated).
 */

[aria-disabled="true"],
[disabled] {
    cursor: not-allowed;
}

/* ============================================================ *
   ELEMENTS
 * ============================================================ */

:host {
    display: block;
    width: 100%;
    margin-bottom: 1rem;
}

label {
    display: block;
}

.content-wrapper {
    display: flex;
    flex-flow: column;
}

.input-wrapper {
    position: relative;
    width: 100%;
    display: flex;
    flex-wrap: wrap;
    padding: var(--forms-padding);
    min-height: var(--forms-height);
    border-style: var(--forms-border-style);
    border-color: var(--forms-border-color);
    border-width: var(--forms-border-width);
    border-radius: var(--forms-border-radius);
    margin-top: var(--forms-border-shift);
    outline: 0;
    background: var(--forms-background);
    color: inherit;
    transition: var(--transition-property) var(--transition-duration) var(--transition-timing-function);
}

textarea {
    width: 100%;
    margin: 0;
    padding: 0;
    border: 0;
    outline: 0;
    font-family: inherit;
    font-size: 100%;
    line-height: var(--line-height-base);
    color: inherit;
    background: transparent;
}

.error-text {
    color: var(--color-status-error);
    padding: 0 var(--forms-padding-horizontal);
}

.help {
    font-size: var(--forms-help-font-size);
    color: var(--forms-color-help);
    font-style: var(--forms-help-font-style);
    padding: 0 var(--forms-padding-horizontal);
    display: inline;
}

/* -------------------- VARIATION -- label-left -------------------- */
:host([label-position="left"]) label,
:host([label-position="left"]) c-label {
    display: flex;
    flex-wrap: wrap;
    align-items: baseline;
    gap: 0 var(--forms-label-left-label-spacing);
}

:host([label-position="left"]) .label-text {
    flex-basis: var(--forms-label-left-label-size);
}

:host([label-position="left"]) .content-wrapper {
    flex-grow: 1000;
    flex-basis: var(--forms-label-left-input-size);
}

/* -------------------- VARIATION -- label-inside -------------------- */
:host([label-position="inside"]) .input-wrapper {
    display: flex;
    align-items: baseline;
}

.input-wrapper::before {
    --grid-height: calc(var(--line-height-base) * 1rem);

    /* Show a grid based on the root font size */
    content: "";
    position: absolute;
    inset: var(--grid-height) 0 2px 0;
    background-image: url('data:image/svg+xml;charset=UTF-8,<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100"><rect stroke="%23999" stroke-width="5" stroke-opacity="0.2" fill="transparent" width="100" height="1"/></svg>');
    background-repeat: repeat;
    background-size: var(--grid-height);
    display: var(--forms-textarea-lines-display, none);
    margin: var(--forms-padding-vertical);
}

/* -------------------- STATES -------------------- */
/* stylelint-disable no-descending-specificity */
.input-wrapper:hover {
    border-color: var(--forms-border-color-hover);
}

.input-wrapper:focus-within {
    outline: var(--forms-outline);
    outline-offset: var(--forms-outline-offset);
}

/* Error */
.error-text,
.error .label-text,
.error .input-wrapper::after {
    color: var(--color-status-error);
}

.error .input-wrapper {
    border-color: var(--color-status-error);
    padding-right: 1rem;
}
/* stylelint-enable no-descending-specificity */

.error .input-wrapper::after {
    content: "!";
    position: absolute;
    right: var(--forms-padding-horizontal);
    font-weight: 900;
}

/* Disabled */
.disabled {
    color: var(--forms-color-disabled);
}

/* stylelint-disable no-descending-specificity */
.disabled .input-wrapper {
    border-color: var(--forms-border-color-disabled);
    background: var(--forms-background-disabled);
    padding: var(--forms-padding-disabled);
}

.disabled .input-wrapper:hover {
    border-color: var(--forms-border-color-disabled);
}
/* stylelint-enable no-descending-specificity */`
/* eslint-enable max-len */

/*
 * Define web component tag for c-textarea.
 */

window.customElements.define('c-textarea', CTextareaComponent)

/*
 * Default export the concrete class for c-textarea.
 */

export default CTextareaComponent
