/* eslint max-classes-per-file: off, class-methods-use-this: off, max-lines: off */

import BaseComponent from '../base_component.js?v=1.12.1'

/*
 * Custom web component script.
 * This is generated by the builder and loaded from the component's `script.js`.
 */

class StyleguideHelperComponent extends BaseComponent {

    /**
     * Get the pretty-formatted markup of a HTML element.
     *
     * @param   {HTMLElement} element   A HTML element
     *
     * @return  {String}                The formatted markup
     */

    static getMarkup(element) {
        const markup = element.outerHTML
            .replaceAll('&quot;', '"')
            .replaceAll('=""', '')
            .replaceAll('>', '>\n')
            .replaceAll('<', '\n<')

        const lines        = []
        const indentSpaces = 4
        let indent         = 0

        markup.split('\n').forEach(line => {
            const lineTrimmed = line.trim()

            if (!lineTrimmed)
                return

            if (lineTrimmed.startsWith('</'))
                indent -= indentSpaces

            lines.push(lineTrimmed.padStart(lineTrimmed.length + indent, ' '))

            if (!lineTrimmed.startsWith('</') && lineTrimmed.endsWith('>'))
                indent += indentSpaces
        })

        return lines.join('\n')
    }

    render() {
        super.render()
        const [slotElement] = this.shadowRoot.querySelector('slot').assignedElements()
        const markup        = StyleguideHelperComponent.getMarkup(slotElement)

        if (this.data)
            slotElement.updateData(this.data)

        this.shadowRoot.querySelector('#markup').textContent = markup
    }

}

/*
 * Register the event listener for the collapsing toggle button.
 */

document.addEventListener('styleguideToggleCollapsable', event => {
    const collapsableItem = event.info.component.shadowRoot.getElementById('collapsable')

    if (collapsableItem.getAttribute('isCollapsed') === 'false')
        collapsableItem.setAttribute('isCollapsed', '')
    else
        collapsableItem.toggleAttribute('isCollapsed')
})

/*
 * HTML template & CSS style for styleguide-helper.
 * This is automatically generated by the builder.
 */

/* eslint-disable max-len */
StyleguideHelperComponent.template = `<article>
    <section>
        <div class="component-content">
            <slot>Component</slot>
        </div>
        <c-tag><slot name="info">Component</slot></c-tag>
    </section>

    <section id="collapsable" class="collapsable-section" isCollapsed="\${attributes.stacked || false}">
        <h3><a t-on="click:styleguideToggleCollapsable" class="toggle">Integration</a></h3>
        <div class="collapsable">
            <div class="collapsable-content">
                <c-tag>Markup</c-tag>
                <code id="markup">Markup</code>
            </div>

            <div class="collapsable-content">
                <c-tag>Data</c-tag>
                <code id="data">\${data ? JSON.stringify(data, null, 4) : ''}</code>
            </div>
        </div>
    </section>
</article>`

StyleguideHelperComponent.style = `:host {
    --collapsable-border-width: 1px;
    --colapsable-border: var(--collapsable-border-width) solid var(--color-ui-2);
    --collapsable-spacing: 1rem;

    margin: 0 calc(-1 * var(--collapsable-spacing));
}

article {
    margin-bottom: 5rem;
    display: flex;
    flex-wrap: wrap;
}

section {
    border: var(--colapsable-border);
    display: flex;
    flex-wrap: wrap;
    flex: 1 0;
    align-content: baseline;
    margin: 0 calc(-1 * var(--collapsable-border-width)) calc(-1 * var(--collapsable-border-width)) 0;
    min-width: 300px;
    transition: all var(--transition-duration) var(--transition-timing-function);
}

:host([stacked]) section {
    flex-basis: 100%;
}

c-tag { /* stylelint-disable-line selector-type-no-unknown */
    text-transform: uppercase;
    display: table;
    order: 1;
    padding-left: var(--collapsable-spacing);
}

h3 {
    margin: 0;
}

code {
    font-family: "Andale Mono", Menlo, Monaco, "Courier New", monospace;
    font-size: 0.875rem;
    white-space: pre-wrap;
    color: var(--color-content-1);
    background: var(--color-ui-1);
    padding: var(--collapsable-spacing);
    display: block;
    transition: all var(--transition-duration) var(--transition-timing-function);
}

/* -------------------- SECTION CONTENT -------------------- */
.component-content {
    display: block;
    order: 2;
    width: 100%;
    padding: var(--collapsable-spacing) var(--collapsable-spacing) 2rem;
}

.collapsable-content {
    flex: 1 1 30%;
    border-top: var(--colapsable-border);
    transition: all var(--transition-duration) var(--transition-timing-function);
    transition-property: border;
}

.collapsable-content:first-child {
    border-right: var(--colapsable-border);
}

/* -------------------- COLLAPSABLE SECTION -------------------- */
.collapsable-section {
    background: var(--color-ui-1);
    border: var(--colapsable-border);
    display: grid;
    grid-template-columns: 1fr;
    grid-template-rows: auto 1fr;
    transition: all var(--transition-duration) var(--transition-timing-function);
}

.collapsable {
    display: flex;
}

/* Toggle Element */
.toggle {
    display: flex;
    justify-content: space-between;
    padding: var(--collapsable-spacing);
    text-transform: uppercase;
}

.toggle::after {
    --size: 1rem;

    content: "△";
    display: inline-block;
    transition: all var(--transition-duration) var(--transition-timing-function);
}

.toggle:hover {
    cursor: pointer;
    opacity: 0.6;
}

.toggle:hover::after {
    content: "▲";
}

/* Collapsed State */
.collapsable-section[isCollapsed] .collapsable {
    display: none;
}

.collapsable-section[isCollapsed="false"] .collapsable {
    display: block;
}

.collapsable-section[isCollapsed] .toggle::after {
    transform: rotate(180deg);
}`
/* eslint-enable max-len */

/*
 * Define web component tag for styleguide-helper.
 */

window.customElements.define('styleguide-helper', StyleguideHelperComponent)

/*
 * Default export the concrete class for styleguide-helper.
 */

export default StyleguideHelperComponent
