const LOGGER_STORAGE_KEY = 'logger'

/**
 * The logger class which can be used to log messages to the JavaScript console.
 *
 * This is basically a wrapper around ``console.…()``, but it adds functionality
 * to handle debug logs by introducing a persistent debug mode.
 *
 * .. hint::
 *
 *      Have a look at :ref:`Debugging components` if you want to know how to
 *      debug components. Also have a look at :js:meth:`debug()` and
 *      :js:meth:`enableDebug()` to see how debug log messages are handled.
 */

class Logger {

    constructor(name) {
        this.name = name
    }

    /**
     * Enable debug mode for one or multiple components.
     *
     * .. hint::
     *
     *      Enabling the debug mode is persistent over site reloads and even
     *      browser restarts. This is achieved by storing the debug component
     *      names in the local storage of the browser.
     *
     * @param   {Array} components  The names of the components to activate debugging for
     *
     * @return  {Array}             The enabled debug components
     */

    static enableDebug(...components) {
        localStorage.setItem(LOGGER_STORAGE_KEY, [
            ...Logger.debugComponents,
            ...components,
        ]
            .filter(component => component)
            .filter((value, index, self) => self.indexOf(value) === index))

        return Logger.updateDebugComponents()
    }

    /**
     * Disable debug mode for one or multiple components.
     *
     * @param   {Array} components  The names of the components to disable debugging for
     *
     * @return  {Array}             The enabled debug components
     */

    static disableDebug(...components) {
        localStorage.setItem(
            LOGGER_STORAGE_KEY,
            Logger.debugComponents.filter(component => !components.includes(component)),
        )

        return Logger.updateDebugComponents()
    }

    /**
     * Reset the debug mode for all components.
     */

    static resetDebug() {
        localStorage.removeItem(LOGGER_STORAGE_KEY)
        Logger.updateDebugComponents()
    }

    /**
     * Update the ``debugComponents`` property of the :js:class:`Logger`.
     */

    static updateDebugComponents() {
        const components       = localStorage.getItem(LOGGER_STORAGE_KEY)
        Logger.debugComponents = components ? components.split(',') : []
        return Logger.debugComponents
    }

    /* eslint-disable no-console */

    /**
     * Log a debug message to the console.
     *
     *
     * .. important::
     *
     *      By default, debug logs are deactivated and **not logged** to the
     *      console at all. This is due to browser performance issues on
     *      extensive logging usage.
     *
     *      Thus, if you want to see debug logs, you've to explicitly activate
     *      them via :js:meth:`enableDebug()`.
     *
     * @param   {Array} message     The logging message
     */

    debug(...message) {
        if (Logger.debugComponents.includes(this.name))
            console.debug('[%s]:', this.name, ...message)
    }

    /**
     * Log an info message to the console.
     *
     * @param   {Array} message     The logging message
     */

    info(...message) {
        console.info('[%s]:', this.name, ...message)
    }

    /**
     * Log a warning message to the console.
     *
     * @param   {Array} message     The logging message
     */

    warn(...message) {
        console.warn('[%s]:', this.name, ...message)
    }

    /**
     * Log an error message to the console.
     *
     * @param   {Array} message     The logging message
     */

    error(...message) {
        console.error('[%s]:', this.name, ...message)
    }

    /* eslint-enable no-console */

}

Logger.updateDebugComponents()

export default Logger
