import {cleanContextEval} from './utils.js?v=1.12.1'

/**
 * Checks if there are attributes that start with a certain name.
 *
 * @param   {String}    name    The name
 *
 * @return  {Boolean}           Flag if there are matching attributes
 */

HTMLElement.prototype.hasAttributesStartingWith = function hasAttributesStartingWith(name) {
    for (const attr of this.attributes) {
        if (attr.name.startsWith(name))
            return true
    }
    return false
}

/**
 * Append multiple child nodes to an element.
 *
 * @param   {Array} children    The child nodes
 */

HTMLElement.prototype.appendChildren = function appendChildren(children) {
    for (const child of children) {
        if (typeof child !== 'undefined' && child !== null)
            this.appendChild(child)
    }
}

/**
 * Get all child nodes of a HTML element.
 *
 * There's already a `children` property on the HTMLElement instance. However,
 * this property will only return the child HTML elements, but not the child
 * nodes. This means that text or comment nodes are missing, thus this function.
 *
 * @return                      The child elements
 */

HTMLElement.prototype.getChildNodes = function getChildNodes() {
    let child    = this.firstChild
    let children = []

    while (child) {
        children = [...children, child.cloneNode(true)]
        child    = child.nextSibling
    }

    return children
}

/**
 * Replace the tag name of the element.
 *
 * The function will create a new element and will not update the current
 * element in place. This is because we're using the `outerHTML` attribute,
 * which cannot be modified in case an element has no parent node.
 *
 * Please take the following examples:
 *
 * .. code-block:: html
 *
 *      <ul></ul>
 *
 *      <ul>
 *          <li>Hello World</li>
 *      </ul>
 *
 *      <ul data-example="Hello">
 *          <li>Hello World</li>
 *      </ul>
 *
 * If you run `ulElement.replaceTagName(`ol`), you'll end up with this:
 *
 * .. code-block:: html
 *
 *      <ol></ol>
 *
 *      <ol>
 *          <li>Hello World</li>
 *      </ol>
 *
 *      <ol data-example="Hello">
 *          <li>Hello World</li>
 *      </ol>
 *
 * @param   {String}        tagName     The new tag name
 *
 * @return  {HTMLElement}               The new HTML element
 */

// eslint-disable-next-line max-statements
HTMLElement.prototype.replaceTagName = function replaceTagName(tagName) {
    let search = '^<[^ >]+'         // Match the first opening tag, e.g. '<ul'
    search += '(?<attrs> [^>]+)?'   // Match the optional attributes, e.g. ' data-example="Hello"'
    search += '>'                   // Match the close bracket of the opening element, i.e. '>'
    search += '(?<children>.+)?'    // Match all optional children, e.g. '<li>Hello World</li>'
    search += '</[^>]+>$'           // Match the last closing tag, e.g. '</ul>'

    const searchRegex  = new RegExp(search, 'msu')
    const match = this.outerHTML.match(searchRegex)

    const element = document.createElement(tagName)
    if (match && match.groups) {
        if (match.groups.attrs) {
            const attrs = match.groups.attrs.trim().split(' ')
            attrs.forEach(attr => {
                const [name, value] = attr.split('=')
                element.setAttribute(name, value ? value.replace(/"/gu, '') : '')
            })
        }

        if (match.groups.children) 
            element.textContent = match.groups.children
    }

    return element
}





/**
 * Interpolate a string with the new ES6 template syntax.
 *
 * @param   {Object}    The interpolate parameters (key-value pairs)
 *
 * @return  {String}    The interpolated string
 */

// eslint-disable-next-line no-extend-native
String.prototype.interpolate = function interpolate(params) {
    return cleanContextEval(`return \`${this}\``, params)
}
