const iconsObject = [
    'applications',
    'arrow-down',
    'arrow-left',
    'arrow-right',
    'arrow-up',
    'back',
    'calendar',
    'check',
    'chevron-down',
    'chevron-left',
    'chevron-right',
    'chevron-up',
    'clock',
    'cross',
    'dash',
    'environment',
    'exclamation-mark',
    'external',
    'flash',
    'flow-1',
    'flow-2',
    'flow-3',
    'folder',
    'forward',
    'gear',
    'hashtag',
    'info',
    'list',
    'location',
    'log-in',
    'log-out',
    'magic-wand',
    'media',
    'moon',
    'node',
    'nodes',
    'patch-run',
    'pause',
    'person',
    'play',
    'plus',
    'restart',
    'restart-2',
    'restart-3',
    'rocket',
    'save',
    'search',
    'sun',
    'thumb-up',
    'thumbnails',
    'trash',
    'user-ok',
    'versions',
    'warning',
]

export default {
    argTypes: {
        href: {
            control: 'text',
            description: 'Link, that spans over the whole block and is indicated by an arrow' +
                ' on the bottom right side of the block.',
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
        },

        icon: {
            control: {type: 'select'},
            description: 'Optional icon displayed on the header.',
            options: iconsObject,
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
        },
        position: {
            control: {type: 'inline-radio'},
            description: 'Optionally align content. (middle = align content vertically, center' +
                ' = align content horizontally and vertically).',
            mapping: {
                default: '',
            },
            options: ['middle', 'center', 'default'],
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
            type: {summary: 'string'},
        },
        ratio: {
            control: {type: 'inline-radio'},
            options: ['2x1', '4x3', '1x1', '3x4', '1x2'],
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
        },
        slotBottom: {
            control: 'text',
            description: 'Content, that is aligned to the bottom of the element.',
            table: {
                category: 'slots',
                type: {summary: 'string'},
            },
        },
        slotDefault: {
            control: 'text',
            description: 'Main Content, can be aligned in the middle or centered via attributes.',
            table: {
                category: 'slots',
                type: {summary: 'string'},
            },
        },
        slotFooter: {
            control: 'text',
            description: 'Content, that is aligned to the bottom of the element and is  styled' +
                ' to be more decent.',
            table: {
                category: 'slots',
                type: {summary: 'string'},
            },
        },
        slotHeader: {
            control: 'text',
            description: 'Content, that is aligned to the top of the element.',
            table: {
                category: 'slots',
                type: {summary: 'string'},
            },
        },
        status: {
            control: {type: 'inline-radio'},
            description: 'Optional status indicator for the block.',
            mapping: {
                default: '',
            },
            options: ['success', 'error', 'running', 'pending', 'approved', 'default'],
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
        },
        tag: {
            control: 'text',
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
        },
        title: {
            control: 'text',
            description: 'Optional title for the block. Will also render a line.',
            table: {
                category: 'attributes',
                type: {summary: 'string'},
            },
        },
    },
    parameters: {
        docs: {
            description: {
                component: 'The `c-block` is a very versatile component. Use it to display ' +
                    'information snippets or listed contents.',
            },
        },
    },
    tags: ['autodocs'],
    title: 'Composed Components/Section and Blocks/Block',
}

/* eslint-disable max-statements */
// eslint-disable-next-line max-len
const Template = ({href, icon, slotBottom, slotDefault, slotFooter, slotHeader, status, tag, title, position, ratio}) => {
    const template = document.createElement('c-block')

    const attributes = { href, icon, position, ratio, status, tag, title }
    for (const [key, value] of Object.entries(attributes)) {
        // eslint-disable-next-line no-undefined
        if (value !== undefined && value !== null && value !== '') 
            template.setAttribute(key, value)
    }

    const range = document.createRange()

    if (slotDefault) {
        const fragment = range.createContextualFragment(slotDefault)
        template.appendChild(fragment)
    }

    if (slotHeader) {
        const headerSlot = document.createElement('div')
        headerSlot.setAttribute('slot', 'header')
        headerSlot.append(...range.createContextualFragment(slotHeader).childNodes)
        template.appendChild(headerSlot)
    }

    if (slotFooter) {
        const footerSlot = document.createElement('div')
        footerSlot.setAttribute('slot', 'footer')
        footerSlot.append(...range.createContextualFragment(slotFooter).childNodes)
        template.appendChild(footerSlot)
    }

    if (slotBottom) {
        const bottomSlot = document.createElement('div')
        bottomSlot.setAttribute('slot', 'bottom')
        bottomSlot.append(...range.createContextualFragment(slotBottom).childNodes)
        template.appendChild(bottomSlot)
    }

    return template
}

/* eslint-enable max-statements */

export const Block = Template.bind({})
Block.args = {
    href: 'http://confirm.ch',
    icon: 'cross',
    position: 'middle',
    ratio: '1x1',
    slotBottom: 'I am content in the Bottom slot.',
    slotDefault: 'I am content in the Default slot.',
    slotFooter: 'I am content in the Footer slot.',
    slotHeader: 'I am content in the Header slot.',
    title: 'I am a title.',
}

export const Empty = Template.bind({})

export const WithLargeIcon = Template.bind({})
WithLargeIcon.args = {
    position: 'center',
    ratio: '1x1',
    slotDefault: '<c-icon icon="calendar" size="lg"></c-icon> ',
    slotFooter: 'I am a block with a large centered icon in the default slot.',
}

export const WithStatus = Template.bind({})
WithStatus.args = {
    slotDefault: 'This block has an error status.',
    status: 'error',
    tag: 'Error',
}

export const WithFormElements = Template.bind({})
WithFormElements.args = {
    position: 'middle',
    ratio: '1x1',
    slotDefault: '<c-input form-skin="default" value="I am an input"></c-input>',
    title: 'I am a title.',
}

export const LinkBlock = Template.bind({})
LinkBlock.args = {
    ratio: '3x2',
    slotDefault: '<c-heading level="5">I am a demo link block</c-heading><br>' +
        '<c-link secondary>A first link</c-link><br/><c-link secondary>A second link</c-link>',
}