/* eslint max-lines: off */

import Cookie from './cookie.js?v=1.12.1'
import Logger from './logging.js?v=1.12.1'

const THEME_COOKIE_NAME      = 'theme'
const THEME_MODE_COOKIE_NAME = 'themeMode'

const THEME_LOGGER = new Logger('theme')

/**
 * The theme class which provides support to access theme settings, and to
 * switch between different themes & the theme mode.
 *
 * .. hint::
 *
 *      The Theme can also be :ref:`accessed through the meta object
 *      <access theme>`.
 */

class Theme {

    /**
     * Log a debug message.
     *
     * @param   {Array} message     The message
     */

    static log(...message) {
        THEME_LOGGER.debug('🔆', ...message)
    }

    /**
     * Initialise the theme executing the following tasks:
     *
     * - Set the project preferences (see :js:meth:`currentTheme()`
     *   and :js:meth:`currentMode()`
     * - Call :js:meth:`initialiseUserPreferences()`
     * - Call :js:meth:`handleEvents()`
     *
     * .. important::
     *
     *      This method is automatically called once, when the document is
     *      loaded.
     */

    static initialise() {
        Theme.projectPreferences = {
            'mode': Theme.getClass('mode'),
            'theme': Theme.getClass('theme'),
        }

        if (!Theme.projectPreferences.theme) {
            const htmlElement = document.getElementsByTagName('html')[0].cloneNode(false)
            htmlElement.classList.add('theme-…')
            THEME_LOGGER.error(
                'Missing project theme, please update <html> element to:',
                htmlElement,
            )
        }

        Theme.initialiseUserPreferences()

        Theme.handleEvents()
    }

    /**
     * Initialise the user preferences.
     */

    static initialiseUserPreferences() {
        if (Theme.userTheme) {
            Theme.log('Initialising user preferred theme')
            Theme.activateTheme(Theme.userTheme, true)
        }

        if (Theme.userMode) {
            Theme.log('Initialising user preferred theme mode')
            Theme.activateMode(Theme.userMode, true)
        }
    }

    /**
     * The currently activated theme.
     *
     * @return  {String}    The current theme
     */

    static get currentTheme() {
        return Theme.getClass('theme')
    }

    /**
     * The currently activated mode.
     *
     * @return  {String}    The current mode
     */

    static get currentMode() {
        return Theme.getClass('mode') ||
            (window.matchMedia('(prefers-color-scheme: dark)').matches ? 'dark' : 'light')
    }

    /**
     * The user-preferred theme.
     *
     * @return  {String}    The user-preferred theme
     */

    static get userTheme() {
        return Cookie.get(THEME_COOKIE_NAME)
    }

    /**
     * The user-preferred mode.
     *
     * @return  {String}    The user-preferred mode
     */

    static get userMode() {
        return Cookie.get(THEME_MODE_COOKIE_NAME)
    }

    /**
     * Get a CSS class with a certain prefix.
     *
     * When talking about prefixes, then we talk about the following format:
     *
     * .. code-block::
     *
     *      {prefix}-{value}
     *
     * @param   {String} prefix     The prefix
     *
     * @return  {String}            The current value
     */

    static getClass(prefix) {
        for (const item of document.getElementsByTagName('html')[0].classList) {
            if (item.startsWith(`${prefix}-`))
                return item.replace(`${prefix}-`, '')
        }

        return null
    }

    /**
     * Remove a CSS class with a certain prefix.
     *
     * When talking about prefixes, then we talk about the following format:
     *
     * .. code-block::
     *
     *      {prefix}-{value}
     *
     * @param   {String} prefix     The prefix
     */

    static removeClass(prefix) {
        document.getElementsByTagName('html')[0].classList
            .remove(`${prefix}-${Theme.getClass(prefix)}`)
    }

    /**
     * Replace a CSS class with a certain prefix.
     *
     * When talking about prefixes, then we talk about the following format:
     *
     * .. code-block::
     *
     *      {prefix}-{value}
     *
     * @param   {String} prefix     The prefix
     * @param   {String} value      The new value
     */

    static replaceClass(prefix, value) {
        const current     = Theme.getClass(prefix)
        const {classList} = document.querySelector('html')
        const newClass    = `${prefix}-${value}`

        if (current)
            classList.replace(`${prefix}-${current}`, newClass)
        else
            classList.add(newClass)
    }

    /**
     * Load and activate a specific theme.
     *
     * @param   {String} theme          The theme name
     * @param   {Boolean} ephemeral     Do not store the theme persistently
     */

    static activateTheme(theme, ephemeral) {
        if (ephemeral)
            Theme.log(`Activating "${theme}" theme ephemerally`)
        else {
            Theme.log(`Activating "${theme}" theme persistently`)
            Cookie.set(THEME_COOKIE_NAME, theme)
        }

        Theme.replaceClass('theme', theme)
    }

    /**
     * Activate a specific theme mode.
     *
     * @param   {String} mode           The theme mode [light|dark]
     * @param   {Boolean} ephemeral     Do not store the theme mode persistently
     */

    static activateMode(mode, ephemeral) {
        if (ephemeral)
            Theme.log(`Activating "${mode}" theme mode ephemerally`)
        else {
            Theme.log(`Activating "${mode}" theme mode persistently`)
            Cookie.set(THEME_MODE_COOKIE_NAME, mode)
        }

        Theme.replaceClass('mode', mode)
    }

    /**
     * Toggle the theme mode between light and dark.
     */

    static toggleMode() {
        Theme.log('Toggling theme mode')
        Theme.activateMode(Theme.currentMode === 'dark' ? 'light' : 'dark')
    }

    /**
     * Handle all events sent by the following theme-related components:
     *
     * - ``ThemeSwitcher``
     * - ``ThemeModeSwitcher``
     * - ``ThemePreferencesResetter``
     */

    static handleEvents() {
        document.addEventListener('switchTheme', event => {
            Theme.activateTheme(event.info.target.data)
        })

        document.addEventListener('resetTheme', () => {
            Theme.resetPreferences()
        })

        document.addEventListener('switchThemeMode', () => {
            Theme.toggleMode()
        })

        document.addEventListener('resetThemeMode', () => {
            Theme.resetPreferences()
        })
    }

    /**
     * Reset the user preferences and initialise the theme again.
     */

    static resetPreferences() {
        Theme.log('Resetting user theme preferences')

        Cookie.delete(THEME_COOKIE_NAME)
        Cookie.delete(THEME_MODE_COOKIE_NAME)

        if (Theme.projectPreferences.theme)
            Theme.replaceClass('theme', Theme.projectPreferences.theme)
        else
            Theme.removeClass('theme')

        if (Theme.projectPreferences.mode)
            Theme.replaceClass('mode', Theme.projectPreferences.mode)
        else
            Theme.removeClass('mode')
    }

}

Theme.initialise()

export default Theme
